package pps.core.system.provider.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SystemUserRoleRelService;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewInput;
import pps.cloud.system.service.data.sys_organization.GetSysOrganizationViewOutput;
import pps.cloud.system.service.data.user_role.GetSysUserRoleRelInput;
import pps.cloud.system.service.data.user_role.GetSysUserRoleRelOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.MediaTypes;
import pps.core.common.session.PpsUserSession;
import pps.core.common.session.SysOrganization;
import pps.core.system.config.LoginConfig;
import pps.core.system.constant.OuLevelConstant;
import pps.core.system.entity.SysAuthOuScopeView;
import pps.core.system.entity.SysOrganizationRelEnt;
import pps.core.system.entity.SysUserEnt;
import pps.core.system.mapper.SysAuthOuScopeViewMapper;
import pps.core.system.mapper.SysOrganizationRelMapper;
import pps.core.system.mapper.SysUserMapper;
import xstartup.annotation.XImplement;
import xstartup.base.XContext;
import xstartup.base.XUser;
import xstartup.base.util.XCopyUtils;
import xstartup.core.base.provider.spec.XSessionInjectProvider;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.*;
import java.util.stream.Collectors;

import static pps.core.common.constant.BusinessConstant.LOGIN_FLAG_SIMULATE;

@XImplement
public class PpsUserSessionInjectProviderImpl implements XSessionInjectProvider {
    @Override
    public XSingleResult<Object> injectSession(XContext context, XUser user) {
        context.log().info("inject session, userCode:{}", user.getCode());
        SysUserMapper mapper = context.getBean(SysUserMapper.class);
        SysUserEnt sysUserEnt = mapper.selectOne(new LambdaQueryWrapper<SysUserEnt>()
                .eq(SysUserEnt::getId, user.getCode()));
        PpsUserSession session = new PpsUserSession();
        XCopyUtils.copyObject(sysUserEnt, session);
        //获取页面授权信息
        SysAuthOuScopeViewMapper scopeViewMapper = context.getBean(SysAuthOuScopeViewMapper.class);
        SysAuthOuScopeView view = new SysAuthOuScopeView();
        view.setUserId(user.getCode());
        List<SysAuthOuScopeView> sysAuthOuScopeViews = scopeViewMapper.selectList(view);
        Map<String, List<SysAuthOuScopeView>> collect = sysAuthOuScopeViews.stream().collect(Collectors.groupingBy(SysAuthOuScopeView::getMenuId));
        Iterator<Map.Entry<String, List<SysAuthOuScopeView>>> iterator = collect.entrySet().iterator();
        Map<String, List<SysOrganization>> pageOrgMap = new HashMap<>();
        while (iterator.hasNext()) {
            Map.Entry<String, List<SysAuthOuScopeView>> next = iterator.next();
            pageOrgMap.put(next.getKey(), XCopyUtils.copyNewList(next.getValue(), SysOrganization.class));
        }
        session.setPageOrgMap(pageOrgMap);
        GetSysOrganizationViewInput input = new GetSysOrganizationViewInput();
        input.setUserId(session.getId());
        XSingleResult<GetSysOrganizationViewOutput> orgInfo = context.getBean(SysOrganizationCloudService.class).getOuByUserId(context, input);
        session.setOuId(orgInfo.getResult().getOuId());
        session.setOuPath(orgInfo.getResult().getOuPath());
        session.setOuName(orgInfo.getResult().getOuName());
        session.setOuPath(orgInfo.getResult().getOuPath());
        session.setOuLevel(orgInfo.getResult().getOuLevel());
        // 2022-09-29 15:14:47, 查询用户所属的非机关组织机构
        GetSysOrganizationViewOutput ownerOrg = findOwnerOrg(context, orgInfo.getResult());
        if (ObjectUtil.isNotNull(ownerOrg)) {
            session.setOwnerOrgId(ownerOrg.getOuId());
            if (CharSequenceUtil.isBlank(ownerOrg.getOuId())) {
                session.setOwnerOrgId(ownerOrg.getId());
            }
            session.setOwnerOrgName(ownerOrg.getOuName());
            session.setOwnerOrgPath(ownerOrg.getOuPath());
            session.setOwnerOrgLevel(ownerOrg.getOuLevel());
            if (CharSequenceUtil.isBlank(ownerOrg.getOuPath())) {
                //路径为空 查询
                SysOrganizationRelMapper bean = context.getBean(SysOrganizationRelMapper.class);
                List<SysOrganizationRelEnt> sysOrganizationRelEnts = bean.selectList(new LambdaQueryWrapper<SysOrganizationRelEnt>()
                        .eq(SysOrganizationRelEnt::getOuId, session.getOwnerOrgId())
                        .le(SysOrganizationRelEnt::getEffectTime, new Date())
                        .ge(SysOrganizationRelEnt::getEndTime, new Date()));
                if (CollUtil.isNotEmpty(sysOrganizationRelEnts)) {
                    session.setOwnerOrgPath(sysOrganizationRelEnts.get(0).getOuIdPath());
                }
            }
        }
        //获取当前用户所属组织机构的介质类型和当前用户授权的组织机构（最终获取组织机构的介质类型）
        if (CharSequenceUtil.isNotBlank(session.getId())) {
            session.setMediaTypes(CollUtil.list(false, MediaTypes.builder()
                    .MediaTypeCode("GAS")
                    .MediaTypeName("天然气")
                    .sort(2)
                    .build()));
        }
        if (user.getOrgId().equals(LOGIN_FLAG_SIMULATE)) {
            session.setDevops(1);
        } else {
            SystemUserRoleRelService roleRelService = context.getBean(SystemUserRoleRelService.class);
            GetSysUserRoleRelInput roleRelInput = new GetSysUserRoleRelInput();
            roleRelInput.setUserId(session.getId());
            XListResult<GetSysUserRoleRelOutput> roleRelResult = roleRelService.getSysRoleByUserId(context, roleRelInput);
            roleRelResult.throwIfFail();
            List<String> roleList = roleRelResult.getResult().stream().map(GetSysUserRoleRelOutput::getRoleId).collect(Collectors.toList());
            String value = LoginConfig.adminRoles.findValue(context);
            session.setDevops(0);
            if (CharSequenceUtil.isNotBlank(value)) {
                roleList.stream().filter(value::contains).findFirst().ifPresent(role -> session.setDevops(1));
            }
        }
        session.setIsSuperAdmin(BusinessConstant.ZERO);
        session.setEnv(context.getEnv().getText());
        SystemUserRoleRelService roleRelService = context.getBean(SystemUserRoleRelService.class);
        GetSysUserRoleRelInput roleRelInput = new GetSysUserRoleRelInput();
        roleRelInput.setUserId(session.getId());
        XListResult<GetSysUserRoleRelOutput> roleRelResult = roleRelService.getSysRoleByUserId(context, roleRelInput);
        roleRelResult.throwIfFail();
        List<String> roleList = roleRelResult.getResult().stream().map(GetSysUserRoleRelOutput::getRoleId).collect(Collectors.toList());
        String value = LoginConfig.adminRoles.findValue(context);
        if (CharSequenceUtil.isNotBlank(value)) {
            roleList.stream().filter(value::contains).findFirst().ifPresent(role -> session.setIsSuperAdmin(BusinessConstant.ONE));
        }
        return XSingleResult.success(session);
    }

    // 查找所属机构
    private GetSysOrganizationViewOutput findOwnerOrg(XContext context, GetSysOrganizationViewOutput org) {
        if (CharSequenceUtil.equals(OuLevelConstant.JI_GUAN_CHU_SHI.getCode(), org.getOuLevel()) ||
                CharSequenceUtil.equals(OuLevelConstant.JI_GUAN_KE_SHI.getCode(), org.getOuLevel())) {
            //查找上级机构
            List<String> idPathList = CharSequenceUtil.split(org.getOuPath(), '/');
            if (CollUtil.isEmpty(idPathList)) {
                return null;
            } else {
                //查找上级机构
                String pareOuId = idPathList.get(idPathList.size() - 2);
                SysOrganizationCloudService organizationService = context.getBean(SysOrganizationCloudService.class);
                GetSysOrganizationViewInput input = new GetSysOrganizationViewInput();
                input.setId(pareOuId);
                XSingleResult<GetSysOrganizationViewOutput> sysOrganizationById = organizationService.getSysOrganizationById(context, input);
                if (sysOrganizationById.isSuccess()) {
                    return findOwnerOrg(context, sysOrganizationById.getResult());
                } else {
                    return null;
                }
            }
        } else {
            return org;
        }
    }
}