package pps.core.base.excel;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.base.entity.BasePhotovoltaicPlantView;
import pps.core.base.entity.PlantPredictedPowerDataEnt;
import pps.core.base.entity.ThirdWeatherDataEnt;
import pps.core.base.entity.ThirdWeatherDataView;
import pps.core.base.mapper.*;
import pps.core.base.service.data.third_weather_data.ThirdApiWeatherExcelData;
import pps.core.base.service.data.third_weather_data.WeatherDeleteCondition;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import xstartup.base.XContext;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 天气数据监听器
 *
 * @author ZWT
 * @date 2024/04/30 11:04
 */
public class ThirdApiWeatherDataListener implements ReadListener<ThirdApiWeatherExcelData> {

    private XContext context;
    /**
     * 缓存的数据
     */
    private List<ThirdApiWeatherExcelData> cachedDataList = new ArrayList<>(4096);

    public ThirdApiWeatherDataListener(XContext context) {
        this.context = context;
    }

    @Override
    public void invoke(ThirdApiWeatherExcelData thirdApiWeatherData, AnalysisContext analysisContext) {
        cachedDataList.add(thirdApiWeatherData);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        this.saveHistory(context, cachedDataList);
    }

    private void saveHistory(XContext context, List<ThirdApiWeatherExcelData> cachedDataList) {
        DateTime date = DateUtil.date();
        //查第二采油厂所有电站
        BasePhotovoltaicPlantViewMapper viewMapper = context.getBean(BasePhotovoltaicPlantViewMapper.class);
        List<BasePhotovoltaicPlantView> plantList = viewMapper.selectPlantList(BasePhotovoltaicPlantView.builder()
                .ouId("2c4c9d1c-ad55-4c14-b889-4dc33e3461e2")
                .build());
        //新数据删除接收
        List<WeatherDeleteCondition> deleteList = new ArrayList<>(32);
        List<ThirdWeatherDataView> weatherDataViewList = new ArrayList<>(2048);
        //todo 老表逻辑(后期删除):后续可能替换
        List<PlantPredictedPowerDataEnt> oldDeleteList = new ArrayList<>(32);
        List<PlantPredictedPowerDataEnt> oldBatchList = new ArrayList<>(13312);
        BigDecimal longitude = new BigDecimal("108.321784");
        BigDecimal latitude = new BigDecimal("36.485083");
        //获取开始结束时间,添加删除条件
        Date beginTime = DateUtil.offsetMinute(DateUtil.parseDateTime(cachedDataList.get(0).getTime()), -5);
        Date endTime = DateUtil.offsetMinute(DateUtil.parseDateTime(cachedDataList.get(cachedDataList.size() - 1).getTime()), 15);
        deleteList.add(
                WeatherDeleteCondition.builder()
                        .longitude(longitude)
                        .latitude(latitude)
                        .beginTime(beginTime)
                        .endTime(endTime)
                        .build()
        );
        //todo 老表逻辑(后期删除):删除数据逻辑
        for (BasePhotovoltaicPlantView plantView : plantList) {
            oldDeleteList.add(PlantPredictedPowerDataEnt.builder()
                    .plantId(plantView.getId())
                    .createTime(beginTime)
                    .endTime(endTime)
                    .build());
        }
        //天气数据处理
        for (ThirdApiWeatherExcelData excel : cachedDataList) {
            DateTime time = DateUtil.parseDateTime(excel.getTime());
            ThirdWeatherDataView thirdWeatherDataView = ThirdWeatherDataView.builder()
                    .id(BaseUtils.randomUUIDString())
                    .dni(excel.getDni())
                    .ghi(excel.getGhi())
                    .dhi(excel.getDhi())
                    .forecastTime(time)
                    .tem(excel.getTem())
                    .rhu(excel.getRhu())
                    .wns(excel.getWns())
                    .wnd(excel.getWnd())
                    .wnsGrd(excel.getWnsGrd())
                    .gust(excel.getGust())
                    .pre(excel.getPre())
                    .prsQfe(excel.getPrsQfe())
                    .vis(excel.getVis())
                    .clo(excel.getClo())
                    .uvb(excel.getUvb())
                    .uvi(0)
                    .phs(excel.getPhs())
                    .wep(excel.getWep())
                    .wnd100m(excel.getWnd100m())
                    .wns100m(excel.getWns100m())
                    .wnsGrd100m(excel.getWnsGrd100m())
                    .longitude(longitude)
                    .latitude(latitude)
                    .createTime(date)
                    .updateTime(date)
                    .build();
            weatherDataViewList.add(
                    thirdWeatherDataView
            );
            //todo 老表逻辑(后期删除):封装预测数据
            for (BasePhotovoltaicPlantView plantView : plantList) {
                this.addPlantPredictedPower(oldBatchList, plantView.getId(), thirdWeatherDataView);
            }
        }
        //todo 老表逻辑(后期删除):删除/新增预测数据
        this.deletePlantPredictedPowerList(context, oldDeleteList);
        this.insertBatchPlantPredictedPowerList(context, oldBatchList);
        if (CollUtil.isNotEmpty(deleteList)) {
            ThirdWeatherDataMapper dataMapper = context.getBean(ThirdWeatherDataMapper.class);
            if (deleteList.size() > BaseUtils.BATCH_SIZE) {
                List<List<WeatherDeleteCondition>> subList = BaseUtils.getSubList(deleteList);
                for (List<WeatherDeleteCondition> list : subList) {
                    LambdaQueryWrapper<ThirdWeatherDataEnt> wrapper = new LambdaQueryWrapper<>();
                    for (WeatherDeleteCondition dataEnt : list) {
                        wrapper.or(w -> w.eq(ThirdWeatherDataEnt::getLatitude, dataEnt.getLatitude())
                                .eq(ThirdWeatherDataEnt::getLongitude, dataEnt.getLongitude())
                                .between(ThirdWeatherDataEnt::getForecastTime, dataEnt.getBeginTime(), dataEnt.getEndTime())
                        );
                    }
                    dataMapper.delete(wrapper);
                }
            } else {
                LambdaQueryWrapper<ThirdWeatherDataEnt> wrapper = new LambdaQueryWrapper<>();
                for (WeatherDeleteCondition dataEnt : deleteList) {
                    wrapper.or(w -> w.eq(ThirdWeatherDataEnt::getLatitude, dataEnt.getLatitude())
                            .eq(ThirdWeatherDataEnt::getLongitude, dataEnt.getLongitude())
                            .between(ThirdWeatherDataEnt::getForecastTime, dataEnt.getBeginTime(), dataEnt.getEndTime())
                    );
                }
                dataMapper.delete(wrapper);
            }
        }
        if (CollUtil.isNotEmpty(weatherDataViewList)) {
            ThirdWeatherDataViewMapper dataViewMapper = context.getBean(ThirdWeatherDataViewMapper.class);
            if (weatherDataViewList.size() > BaseUtils.BATCH_SIZE) {
                List<List<ThirdWeatherDataView>> subList = BaseUtils.getSubList(weatherDataViewList);
                subList.forEach(dataViewMapper::insertBatch);
            } else {
                dataViewMapper.insertBatch(weatherDataViewList);
            }
        }
    }

    /**
     * 添加电厂预测功率
     *
     * @param batchList            批处理列表
     * @param plantId              植物id
     * @param thirdWeatherDataView 第三天气数据视图
     */
    private void addPlantPredictedPower(List<PlantPredictedPowerDataEnt> batchList, String plantId, ThirdWeatherDataView thirdWeatherDataView) {
        DateTime date = DateUtil.date(thirdWeatherDataView.getForecastTime());
        batchList.add(PlantPredictedPowerDataEnt.builder()
                .id(BaseUtils.randomUUIDString())
                .plantId(plantId)
                .createTime(thirdWeatherDataView.getCreateTime())
                .yearTime(String.valueOf(date.year()))
                .monthTime(this.zeroFill(date.monthBaseOne()))
                .dayTime(this.zeroFill(date.dayOfMonth()))
                .hourTime(this.zeroFill(date.hour(true)))
                .minTime(this.zeroFill(date.minute()))
                .dataDate(date.toString(BusinessConstant.DATE_FORMAT_ALL))
                .temperature(thirdWeatherDataView.getTem())
                .windSpeed(thirdWeatherDataView.getWns100m())
                .windDirection(new BigDecimal(thirdWeatherDataView.getWnd()))
                .pressure(thirdWeatherDataView.getPrsQfe())
                .humidity(thirdWeatherDataView.getRhu())
                .planeIrradiance(thirdWeatherDataView.getDni())
                .horizontalIrradiance(thirdWeatherDataView.getGhi())
                .power(BigDecimal.ZERO)
                .build());
    }

    /**
     * 条件删除预测数据
     *
     * @param context    上下文
     * @param deleteList 删除列表
     */
    private void deletePlantPredictedPowerList(XContext context, List<PlantPredictedPowerDataEnt> deleteList) {
        if (CollUtil.isNotEmpty(deleteList)) {
            PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
            int size = deleteList.size();
            context.getLogger().info("------ deletePlantPredictedPowerList batchDelete delete size: {}", size);
            if (size > BaseUtils.BATCH_SIZE) {
                List<List<PlantPredictedPowerDataEnt>> subList = BaseUtils.getSubList(deleteList);
                for (List<PlantPredictedPowerDataEnt> list : subList) {
                    LambdaQueryWrapper<PlantPredictedPowerDataEnt> wrapper = new LambdaQueryWrapper<>();
                    for (PlantPredictedPowerDataEnt dataEnt : list) {
                        wrapper.or(w -> w.eq(PlantPredictedPowerDataEnt::getPlantId, dataEnt.getPlantId())
                                .between(PlantPredictedPowerDataEnt::getDataDate, dataEnt.getCreateTime(), dataEnt.getEndTime())
                        );
                    }
                    mapper.delete(wrapper);
                }
            } else {
                LambdaQueryWrapper<PlantPredictedPowerDataEnt> wrapper = new LambdaQueryWrapper<>();
                for (PlantPredictedPowerDataEnt dataEnt : deleteList) {
                    wrapper.or(w -> w.eq(PlantPredictedPowerDataEnt::getPlantId, dataEnt.getPlantId())
                            .between(PlantPredictedPowerDataEnt::getDataDate, dataEnt.getCreateTime(), dataEnt.getEndTime())
                    );
                }
                mapper.delete(wrapper);
            }
        }
    }

    /**
     * 插入间歇装置预测功率列表
     *
     * @param context   上下文
     * @param batchList 批处理列表
     */
    private void insertBatchPlantPredictedPowerList(XContext context, List<PlantPredictedPowerDataEnt> batchList) {
        if (CollUtil.isNotEmpty(batchList)) {
            PlantPredictedPowerDataViewMapper dataViewMapper = context.getBean(PlantPredictedPowerDataViewMapper.class);
            int size = batchList.size();
            context.getLogger().info("------ insertBatchPlantPredictedPowerList batchList insert size: {}", size);
            if (size > BaseUtils.BATCH_SIZE) {
                List<List<PlantPredictedPowerDataEnt>> subList = BaseUtils.getSubList(batchList);
                subList.forEach(dataViewMapper::insertBatch);
            } else {
                dataViewMapper.insertBatch(batchList);
            }
        }
    }

    /**
     * 零填充
     *
     * @param number 数字
     * @return {@link String}
     */
    private String zeroFill(int number) {
        if (number < 10) {
            return "0" + number;
        }
        return String.valueOf(number);
    }
}
