package pps.core.base.excel;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.base.entity.ThirdWindPowerGenerationEnt;
import pps.core.base.entity.WindPredictionFutureView;
import pps.core.base.entity.WindPredictionHistoryView;
import pps.core.base.mapper.ThirdWindPowerGenerationMapper;
import pps.core.base.mapper.WindPredictionFutureViewMapper;
import pps.core.base.mapper.WindPredictionHistoryViewMapper;
import pps.core.base.service.data.third_weather_data.ThirdApiWeatherExcelData;
import pps.core.common.utils.BaseUtils;
import xstartup.base.XContext;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 天气数据监听器(测试用)
 *
 * @author ZWT
 * @date 2024/04/30 11:04
 */
public class ThirdApiWeatherDataListener2 implements ReadListener<ThirdApiWeatherExcelData> {

    private XContext context;
    /**
     * 缓存的数据
     */
    private List<ThirdApiWeatherExcelData> cachedDataList = new ArrayList<>(4096);

    public ThirdApiWeatherDataListener2(XContext context) {
        this.context = context;
    }

    @Override
    public void invoke(ThirdApiWeatherExcelData thirdApiWeatherData, AnalysisContext analysisContext) {
        cachedDataList.add(thirdApiWeatherData);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        this.saveHistory(context, cachedDataList);
    }

    private void saveHistory(XContext context, List<ThirdApiWeatherExcelData> cachedDataList) {
        if (CollUtil.isNotEmpty(cachedDataList)) {
            String stationId = "f4816d9e4e134aaba0dbd483775e36b2";
            String stationName = "吉林油田模拟电站";
            ThirdApiWeatherExcelData first = CollUtil.getFirst(cachedDataList);
            ThirdApiWeatherExcelData last = CollUtil.getLast(cachedDataList);
            //每个csv只要第一天
            DateTime beginTime = DateUtil.parseDateTime(first.getTime());
            DateTime endTime = DateUtil.endOfDay(beginTime);
            //查实际信息
            ThirdWindPowerGenerationMapper mapper = context.getBean(ThirdWindPowerGenerationMapper.class);
            List<ThirdWindPowerGenerationEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdWindPowerGenerationEnt>()
                    .eq(ThirdWindPowerGenerationEnt::getStationId, stationId)
                    .between(ThirdWindPowerGenerationEnt::getCollectTime, beginTime, endTime)
                    .orderByAsc(ThirdWindPowerGenerationEnt::getCollectTime)
            );
            Map<Date, ThirdWindPowerGenerationEnt> collect = list.stream()
                    .collect(Collectors.toMap(ThirdWindPowerGenerationEnt::getCollectTime, Function.identity()));
            //封装数据
            List<WindPredictionFutureView> futureList = new ArrayList<>(96);
            List<WindPredictionHistoryView> historyList = new ArrayList<>(96);
            DateTime dateTime;
            ThirdWindPowerGenerationEnt generationEnt;
            for (ThirdApiWeatherExcelData data : cachedDataList) {
                dateTime = DateUtil.parseDateTime(data.getTime());
                if (endTime.compareTo(dateTime) <= 0) {
                    break;
                }
                WindPredictionHistoryView historyView = new WindPredictionHistoryView();
                historyView.setId(BaseUtils.randomUUIDString());
                historyView.setStationId(stationId);
                historyView.setDataTime(dateTime);
                historyView.setWindDirection(data.getWnd());
                historyView.setWindSpeed(data.getGust());
                historyView.setAirTemperature(data.getTem());
                historyView.setHumidity(data.getRhu());
                historyView.setPressure(data.getPrsQfe());
                historyView.setWnd100m(data.getWnd100m());
                historyView.setWns100m(data.getWns100m());
                historyView.setWnsGrd100m(data.getWnsGrd100m());
                WindPredictionFutureView futureView = new WindPredictionFutureView();
                futureView.setId(BaseUtils.randomUUIDString());
                futureView.setStationId(stationId);
                futureView.setDataTime(dateTime);
                futureView.setWindDirection(data.getWnd());
                futureView.setWindSpeed(data.getGust());
                futureView.setAirTemperature(data.getTem());
                futureView.setHumidity(data.getRhu());
                futureView.setPressure(data.getPrsQfe());
                futureView.setActualPower(BigDecimal.ZERO);
                //匹配实际
                if (collect.containsKey(dateTime)) {
                    generationEnt = collect.get(dateTime);
                    futureView.setActualWindSpeed(generationEnt.getActualWindSpeed());
                    historyView.setActualWindSpeed(generationEnt.getActualWindSpeed());
                    historyView.setActualPower(generationEnt.getActualPower());
                    historyView.setActualWindDirection(generationEnt.getAccurateWindDirection());
                } else {
                    futureView.setActualWindSpeed(BigDecimal.ZERO);
                    historyView.setActualWindSpeed(BigDecimal.ZERO);
                    historyView.setActualPower(BigDecimal.ZERO);
                }
                futureList.add(futureView);
                historyList.add(historyView);
            }
            //入库
            if (CollUtil.isNotEmpty(futureList)) {
                WindPredictionFutureViewMapper futureMapper = context.getBean(WindPredictionFutureViewMapper.class);
                if (futureList.size() > BaseUtils.BATCH_SIZE) {
                    List<List<WindPredictionFutureView>> subList = BaseUtils.getSubList(futureList);
                    subList.forEach(futureMapper::insertBatch);
                } else {
                    futureMapper.insertBatch(futureList);
                }
            }
            if (CollUtil.isNotEmpty(historyList)) {
                WindPredictionHistoryViewMapper historyMapper = context.getBean(WindPredictionHistoryViewMapper.class);
                if (historyList.size() > BaseUtils.BATCH_SIZE) {
                    List<List<WindPredictionHistoryView>> subList = BaseUtils.getSubList(historyList);
                    subList.forEach(historyMapper::batchInsert);
                } else {
                    historyMapper.batchInsert(historyList);
                }
            }
        }
    }
}