package pps.core.base.excel;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import pps.core.base.entity.ThirdWindPowerGenerationView;
import pps.core.base.mapper.ThirdWindPowerGenerationViewMapper;
import pps.core.base.service.data.base_excel.WindPredictionExcel2Data;
import pps.core.common.utils.BaseUtils;
import xstartup.base.XContext;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 风资源数据(测试用)
 *
 * @author ZWT
 * @date 2024/08/30 15:46
 */
public class WindPredictionDataListener2 implements ReadListener<WindPredictionExcel2Data> {

    private XContext context;

    /**
     * 缓存的数据
     */
    private List<WindPredictionExcel2Data> cachedDataList = new ArrayList<>(4096);

    public WindPredictionDataListener2(XContext context) {
        this.context = context;
    }

    @Override
    public void invoke(WindPredictionExcel2Data windPredictionExcelData, AnalysisContext analysisContext) {
        cachedDataList.add(windPredictionExcelData);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        this.saveWindPredictionHistory(context, cachedDataList);
    }

    public void saveWindPredictionHistory(XContext context, List<WindPredictionExcel2Data> cachedDataList) {
        if (CollUtil.isNotEmpty(cachedDataList)) {
            List<ThirdWindPowerGenerationView> batchList = new ArrayList<>(cachedDataList.size());
            String stationId = "f4816d9e4e134aaba0dbd483775e36b2";
            String stationName = "吉林油田模拟电站";
            DateTime date;
            DateTime beginOfDay;
            int minute;
            Integer actualWindDirection;
            for (WindPredictionExcel2Data data : cachedDataList) {
                date = DateUtil.date(data.getDataTime());
                beginOfDay = DateUtil.beginOfDay(date);
                minute = date.minute();
                //转换时间
                switch (minute) {
                    case 10:
                        continue;
                    case 20:
                        date = DateUtil.offsetMinute(date, -5);
                        break;
                    case 40:
                        continue;
                    case 50:
                        date = DateUtil.offsetMinute(date, -5);
                        break;
                }
                //转换风向
                BigDecimal accurateWindDirection = data.getAccurateWindDirection();
                if (BigDecimal.valueOf(45).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 0;
                } else if (BigDecimal.valueOf(90).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 45;
                } else if (BigDecimal.valueOf(135).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 90;
                } else if (BigDecimal.valueOf(180).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 135;
                } else if (BigDecimal.valueOf(225).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 180;
                } else if (BigDecimal.valueOf(270).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 225;
                } else if (BigDecimal.valueOf(315).compareTo(accurateWindDirection) > 0) {
                    actualWindDirection = 270;
                } else {
                    actualWindDirection = 315;
                }
                //封装
                batchList.add(
                        ThirdWindPowerGenerationView.builder()
                                .id(BaseUtils.randomUUIDString())
                                .stationName(stationName)
                                .stationId(stationId)
                                .collectTime(date)
                                .actualWindSpeed(data.getActualWindSpeed())
                                .actualPower(data.getActualPower())
                                .actualGeneration(BigDecimal.ZERO)
                                .accurateWindDirection(data.getAccurateWindDirection())
                                .actualWindDirection(actualWindDirection)
                                .inputTime(beginOfDay)
                                .systemSource("SY")
                                .build()
                );
            }
            ThirdWindPowerGenerationViewMapper viewMapper = context.getBean(ThirdWindPowerGenerationViewMapper.class);
            //批量新增
            if (batchList.size() > BaseUtils.BATCH_SIZE) {
                List<List<ThirdWindPowerGenerationView>> subList = BaseUtils.getSubList(batchList);
                subList.forEach(viewMapper::batchInsert);
            } else {
                viewMapper.batchInsert(batchList);
            }
        }
    }
}
