package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.base.entity.BaseEnergyStorageDeviceEnt;
import pps.core.base.entity.BasePowerLineStorageEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BaseEnergyStorageDeviceMapper;
import pps.core.base.mapper.BasePowerLineStorageMapper;
import pps.core.base.service.data.base_energy_storage_device.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 储能设备模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseEnergyStorageDeviceService {

    /**
     * 储能设备模块--新增
     * POST /base/base-energy-storage-device/create-base-energy-storage-device
     * 接口ID：105602202
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105602202
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("储能设备模块--新增")
    @XApiPost
    public XServiceResult createBaseEnergyStorageDevice(XContext context, CreateBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        if (this.checkMakerNumberExists(mapper, input.getMakerNumber(), null)) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BaseEnergyStorageDeviceEnt entity = XCopyUtils.copyNewObject(input, BaseEnergyStorageDeviceEnt.class);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 储能设备模块--修改
     * POST /base/base-energy-storage-device/update-base-energy-storage-device
     * 接口ID：105602203
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105602203
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("储能设备模块--修改")
    @XApiPost
    public XServiceResult updateBaseEnergyStorageDevice(XContext context, UpdateBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        if (this.checkMakerNumberExists(mapper, input.getMakerNumber(), input.getId())) {
            return XServiceResult.error(context, BusinessError.MakerNumberExists);
        }
        BaseEnergyStorageDeviceEnt entity = this.selectOneByDeviceId(input.getId(), mapper);
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 储能设备模块--删除
     * POST /base/base-energy-storage-device/delete-base-energy-storage-device
     * 接口ID：105602204
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105602204
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("储能设备模块--删除")
    @XApiPost
    public XServiceResult deleteBaseEnergyStorageDevice(XContext context, DeleteBaseEnergyStorageDeviceInput input) {
        String storageId = input.getId();
        if (this.checkStorageIsReference(context, storageId)) {
            return XServiceResult.error(context, BusinessError.StorageReference);
        }
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        if (ObjectUtil.isNull(this.selectOneByDeviceId(storageId, mapper))) {
            return XServiceResult.error(context, XError.NotFound);
        }
        BaseEnergyStorageDeviceEnt entity = new BaseEnergyStorageDeviceEnt();
        entity.setId(storageId);
        entity.setIsDeleted(BusinessConstant.ZERO);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 储能设备模块--详情
     * GET /base/base-energy-storage-device/get-base-energy-storage-device
     * 接口ID：105602200
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105602200
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBaseEnergyStorageDeviceOutput}>
     */
    @XText("储能设备模块--详情")
    @XApiGet
    public XSingleResult<GetBaseEnergyStorageDeviceOutput> getBaseEnergyStorageDevice(XContext context, GetBaseEnergyStorageDeviceInput input) {
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        BaseEnergyStorageDeviceEnt entity = this.selectOneByDeviceId(input.getId(), mapper);
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseEnergyStorageDeviceOutput output = XCopyUtils.copyNewObject(entity, GetBaseEnergyStorageDeviceOutput.class);
        return XSingleResult.success(output);
    }

    /**
     * 储能设备模块--分页查询
     * GET /base/base-energy-storage-device/query-base-energy-storage-device
     * 接口ID：105602201
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105602201
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBaseEnergyStorageDeviceOutput}>
     */
    @XText("储能设备模块--分页查询")
    @XApiGet
    public XPageResult<QueryBaseEnergyStorageDeviceOutput> queryBaseEnergyStorageDevice(XContext context, QueryBaseEnergyStorageDeviceInput input) {
        String ouName = input.getOuName();
        String deviceName = input.getDeviceName();
        BaseEnergyStorageDeviceMapper mapper = context.getBean(BaseEnergyStorageDeviceMapper.class);
        QueryWrapper<BaseEnergyStorageDeviceEnt> queryWrapper = new QueryWrapper<>();
        boolean equals = CharSequenceUtil.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId());
        if (CharSequenceUtil.isNotEmpty(input.getLineId())) {
            queryWrapper.lambda()
                    .notExists(!equals, "SELECT 1 FROM BASE_POWER_LINE_STORAGE W WHERE BASE_ENERGY_STORAGE_DEVICE.ID = W.STORAGE_ID AND W.IS_DELETED = 1 AND W.LINE_ID <> {0}", input.getLineId())
                    .notExists(equals, "SELECT 1 FROM BASE_POWER_LINE_STORAGE W WHERE BASE_ENERGY_STORAGE_DEVICE.ID = W.STORAGE_ID AND W.IS_DELETED = 1")
                    .eq(BaseEnergyStorageDeviceEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BaseEnergyStorageDeviceEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .eq(BaseEnergyStorageDeviceEnt::getIsDeleted, BusinessConstant.ONE)
                .like(CharSequenceUtil.isNotEmpty(ouName), BaseEnergyStorageDeviceEnt::getOuName, ouName)
                .like(CharSequenceUtil.isNotEmpty(deviceName), BaseEnergyStorageDeviceEnt::getDeviceName, deviceName)
                .orderByAsc(BaseEnergyStorageDeviceEnt::getOuName, BaseEnergyStorageDeviceEnt::getDeviceName);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseEnergyStorageDeviceOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查储能设备是否被引用
     *
     * @param context   上下文
     * @param storageId 储能设备ID
     * @return boolean
     */
    private boolean checkStorageIsReference(XContext context, String storageId) {
        BasePowerLineStorageMapper mapper = context.getBean(BasePowerLineStorageMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineStorageEnt>()
                .eq(BasePowerLineStorageEnt::getStorageId, storageId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 通过设备ID查询设备信息
     *
     * @param deviceId 设备id
     * @param mapper   映射器
     * @return {@link BaseEnergyStorageDeviceEnt}
     */
    private BaseEnergyStorageDeviceEnt selectOneByDeviceId(String deviceId, BaseEnergyStorageDeviceMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseEnergyStorageDeviceEnt>()
                .eq(BaseModel::getId, deviceId)
        );
    }

    /**
     * 检查编号是否存在
     *
     * @param mapper      映射器
     * @param makerNumber 出场编号
     * @param id          id
     * @return boolean
     */
    private boolean checkMakerNumberExists(BaseEnergyStorageDeviceMapper mapper, String makerNumber, String id) {
        Long count = mapper.selectCount(new LambdaQueryWrapper<BaseEnergyStorageDeviceEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BaseEnergyStorageDeviceEnt::getMakerNumber, makerNumber)
                .ne(CharSequenceUtil.isNotEmpty(id), BaseModel::getId, id)
        );
        return count > 0;
    }
}