package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBasePriceStrategyCloudService;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.core.base.entity.BasePriceStrategyDetailEnt;
import pps.core.base.mapper.BasePriceStrategyDetailMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 市电峰谷配置Cloud模块
 *
 * @author ZWT
 * @date 2023/09/07 14:52
 */
@XService
public class BasePriceStrategyCloudServiceImpl implements IBasePriceStrategyCloudService {

    /**
     * 市电峰谷配置Cloud模块--策略配置详情列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link GetBasePriceStrategyDetailOutput}>
     */
    @Override
    public XListResult<GetBasePriceStrategyDetailOutput> queryStrategyDetailList(XContext context, GetBasePriceStrategyDetailInput input) {
        String periodTypeKey = input.getPeriodTypeKey();
        String lineId = input.getLineId();
        //查明细
        BasePriceStrategyDetailMapper detailMapper = context.getBean(BasePriceStrategyDetailMapper.class);
        List<BasePriceStrategyDetailEnt> detailList = detailMapper.selectList(
                new LambdaQueryWrapper<BasePriceStrategyDetailEnt>()
                        .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                        .eq(CharSequenceUtil.isNotBlank(input.getStrategyMonth()), BasePriceStrategyDetailEnt::getStrategyMonth, input.getStrategyMonth())
                        .in(CollUtil.isNotEmpty(input.getMonthList()), BasePriceStrategyDetailEnt::getStrategyMonth, input.getMonthList())
                        .eq(CharSequenceUtil.isNotBlank(periodTypeKey), BasePriceStrategyDetailEnt::getPeriodTypeKey, periodTypeKey)
                        .apply(CharSequenceUtil.isNotBlank(lineId), "STRATEGY_ID = ( SELECT STRATEGY_ID FROM BASE_POWER_LINE WHERE ID = {0} ) ", lineId)
                        .orderByAsc(BasePriceStrategyDetailEnt::getStartTime)
        );
        List<GetBasePriceStrategyDetailOutput> outputs;
        if (CollUtil.isEmpty(detailList)) {
            outputs = Collections.emptyList();
        } else {
            outputs = new ArrayList<>(detailList.size());
            GetBasePriceStrategyDetailOutput output;
            for (BasePriceStrategyDetailEnt ent : detailList) {
                output = new GetBasePriceStrategyDetailOutput();
                output.setStrategyId(ent.getStrategyId());
                output.setStrategyMonth(ent.getStrategyMonth());
                output.setPeriodTypeKey(ent.getPeriodTypeKey());
                output.setStartTime(ent.getStartTime());
                output.setEndTime(ent.getEndTime());
                output.setOpenTime(this.strToDateTime(ent.getStartTime()));
                output.setCloseTime(this.strToDateTime(ent.getEndTime()));
                output.setElectrovalence(ent.getElectrovalence());
                outputs.add(output);
            }
        }
        return XListResult.success(outputs);
    }

    /**
     * 字符串转日期时间
     *
     * @param dateStr 日期str
     * @return {@link DateTime }
     */
    private DateTime strToDateTime(String dateStr) {
        //转换开始时间和结束时间(开始时间不能为24:00)
        if (CharSequenceUtil.equals(dateStr, BusinessConstant.END_OF_DAY_TIME)) {
            return BusinessConstant.DATE_FLAG;
        } else {
            return DateUtil.parse(dateStr + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
        }
    }
}