package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.core.base.entity.BaseDieselGeneratorEnt;
import pps.core.base.entity.BasePowerLineWellheadEnt;
import pps.core.base.entity.BaseWellheadEnt;
import pps.core.base.enums.BusinessError;
import pps.core.base.mapper.BasePowerLineWellheadMapper;
import pps.core.base.mapper.BaseWellheadMapper;
import pps.core.base.service.data.base_wellhead.*;
import pps.core.base.utils.ServiceUtil;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;
import xstartup.helper.XTransactionHelper;

import java.util.List;


/**
 * 井口配置模块
 *
 * @author ZWT
 * @date 2023/09/08
 */
@XService
public class BaseWellheadService {

    /**
     * 井口配置模块--新增
     * POST /base/base-wellhead/create-base-wellhead
     * 接口ID：105357657
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105357657
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("井口配置模块--新增")
    @XApiPost
    public XServiceResult createBaseWellhead(XContext context, CreateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        if (this.checkWellNumberExists(mapper, input.getWellNumber(), null)) {
            return XServiceResult.error(context, BusinessError.WellNumberExists);
        }
        BaseWellheadEnt entity = XCopyUtils.copyNewObject(input, BaseWellheadEnt.class);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 井口配置模块--修改
     * POST /base/base-wellhead/update-base-wellhead
     * 接口ID：105426967
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105426967
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("井口配置模块--修改")
    @XApiPost
    public XServiceResult updateBaseWellhead(XContext context, UpdateBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        if (this.checkWellNumberExists(mapper, input.getWellNumber(), input.getId())) {
            return XServiceResult.error(context, BusinessError.WellNumberExists);
        }
        BaseWellheadEnt entity = this.selectOneByWellheadId(input.getId(), mapper);
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 井口配置模块--删除
     * POST /base/base-wellhead/delete-base-wellhead
     * 接口ID：105430749
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105430749
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XText("井口配置模块--删除")
    @XApiPost
    public XServiceResult deleteBaseWellhead(XContext context, DeleteBaseWellheadInput input) {
        String wellheadId = input.getId();
        if (this.checkWellheadIsLineReference(context, wellheadId)) {
            return XServiceResult.error(context, BusinessError.WellheadReferenceLine);
        }
        if (this.checkWellheadIsSpaceReference(context, wellheadId)) {
            return XServiceResult.error(context, BusinessError.WellheadReferenceSpace);
        }
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        if (ObjectUtil.isNull(this.selectOneByWellheadId(wellheadId, mapper))) {
            return XServiceResult.error(context, XError.NotFound);
        }
        BaseWellheadEnt entity = new BaseWellheadEnt();
        entity.setId(wellheadId);
        entity.setIsDeleted(BusinessConstant.ZERO);
        BaseUtils.setBaseModelDefault(entity, context.getSession(PpsUserSession.class));
        return XTransactionHelper.begin(context, () -> {
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 井口配置模块--详情
     * GET /base/base-wellhead/get-base-wellhead
     * 接口ID：105325952
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105325952
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetBaseWellheadOutput}>
     */
    @XText("井口配置模块--详情")
    @XApiGet
    public XSingleResult<GetBaseWellheadOutput> getBaseWellhead(XContext context, GetBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        BaseWellheadEnt entity = this.selectOneByWellheadId(input.getId(), mapper);
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseWellheadOutput output = XCopyUtils.copyNewObject(entity, GetBaseWellheadOutput.class);
        return XSingleResult.success(output);
    }

    /**
     * 井口配置模块--分页查询
     * GET /base/base-wellhead/query-base-wellhead
     * 接口ID：105431641
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-105431641
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QueryBaseWellheadOutput}>
     */
    @XText("井口配置模块--分页查询")
    @XApiGet
    public XPageResult<QueryBaseWellheadOutput> queryBaseWellhead(XContext context, QueryBaseWellheadInput input) {
        BaseWellheadMapper mapper = context.getBean(BaseWellheadMapper.class);
        QueryWrapper<BaseWellheadEnt> queryWrapper = new QueryWrapper<>();
        boolean equals = CharSequenceUtil.equals(BusinessConstant.LINE_FLAG_ID, input.getLineId());
        if (CharSequenceUtil.isNotEmpty(input.getLineId())) {
            queryWrapper.lambda()
                    .notExists(!equals, "SELECT 1 FROM BASE_POWER_LINE_WELLHEAD W WHERE BASE_WELLHEAD.ID = W.WELLHEAD_ID AND W.IS_DELETED = 1 AND W.LINE_ID <> {0}", input.getLineId())
                    .notExists(equals, "SELECT 1 FROM BASE_POWER_LINE_WELLHEAD W WHERE BASE_WELLHEAD.ID = W.WELLHEAD_ID AND W.IS_DELETED = 1")
                    .eq(BaseWellheadEnt::getOuId, input.getOuId());
        } else {
            List<String> allListByOuId = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            queryWrapper.lambda()
                    .in(CollUtil.isNotEmpty(allListByOuId), BaseWellheadEnt::getOuId, allListByOuId);
        }
        queryWrapper.lambda()
                .eq(BaseWellheadEnt::getIsDeleted, BusinessConstant.ONE)
                .like(CharSequenceUtil.isNotBlank(input.getWellNumber()), BaseWellheadEnt::getWellNumber, input.getWellNumber())
                .orderByAsc(BaseWellheadEnt::getOuName, BaseWellheadEnt::getWellNumber);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseWellheadOutput.class);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 检查井口是否被线路引用
     *
     * @param context    上下文
     * @param wellheadId 井口ID
     * @return boolean
     */
    private boolean checkWellheadIsLineReference(XContext context, String wellheadId) {
        BasePowerLineWellheadMapper mapper = context.getBean(BasePowerLineWellheadMapper.class);
        Long count = mapper.selectCount(new LambdaQueryWrapper<BasePowerLineWellheadEnt>()
                .eq(BasePowerLineWellheadEnt::getWellheadId, wellheadId)
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
        );
        return count > 0;
    }

    /**
     * 检查井口是否被间开引用
     *
     * @param context    上下文
     * @param wellheadId 井口id
     * @return boolean
     */
    private boolean checkWellheadIsSpaceReference(XContext context, String wellheadId) {
        ISpaceInstitutionDetailCloudService contextBean = context.getBean(ISpaceInstitutionDetailCloudService.class);
        XSingleResult<Long> result = contextBean.getInstitutionWellheadCountByWellheadId(context, wellheadId);
        result.throwIfFail();
        return result.getResult() > 0;
    }

    /**
     * 通过井口ID查询井口信息
     *
     * @param wellheadId 井口id
     * @param mapper     映射器
     * @return {@link BaseDieselGeneratorEnt}
     */
    private BaseWellheadEnt selectOneByWellheadId(String wellheadId, BaseWellheadMapper mapper) {
        return mapper.selectOne(new LambdaQueryWrapper<BaseWellheadEnt>()
                .eq(BaseModel::getId, wellheadId)
        );
    }

    /**
     * 检查井编号存在
     *
     * @param mapper     映射器
     * @param wellNumber 井号
     * @param id         id
     * @return boolean
     */
    private boolean checkWellNumberExists(BaseWellheadMapper mapper, String wellNumber, String id) {
        Long count = mapper.selectCount(new LambdaQueryWrapper<BaseWellheadEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(BaseWellheadEnt::getWellNumber, wellNumber)
                .ne(CharSequenceUtil.isNotEmpty(id), BaseModel::getId, id)
        );
        return count > 0;
    }
}