package pps.core.base.service;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IConfigOilFieldCloudService;
import pps.cloud.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.core.base.entity.ConfigOilFieldEnt;
import pps.core.base.mapper.ConfigOilFieldMapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XSingleResult;

/**
 * 油田配置Cloud模块
 *
 * @author ZWT
 * @date 2024/06/25 14:13
 */
@XService
public class ConfigOilFieldCloudServiceImpl implements IConfigOilFieldCloudService {

    /**
     * 获取当前油田配置
     *
     * @param context 上下文
     * @return {@link XSingleResult }<{@link GetConfigOilFieldOutput }>
     */
    @Override
    public XSingleResult<GetConfigOilFieldOutput> getCurrentConfig(XContext context) {
        ConfigOilFieldMapper mapper = context.getBean(ConfigOilFieldMapper.class);
        ConfigOilFieldEnt ent = mapper.selectOne(new LambdaQueryWrapper<ConfigOilFieldEnt>()
                .eq(ConfigOilFieldEnt::getIsEnable, BusinessConstant.ZERO)
                .last(BaseUtils.getLastLimit())
        );
        GetConfigOilFieldOutput output;
        if (ObjectUtil.isNull(ent)) {
            output = new GetConfigOilFieldOutput();
        } else {
            output = XCopyUtils.copyNewObject(ent, GetConfigOilFieldOutput.class);
        }
        return XSingleResult.success(output);
    }
}