package pps.core.base.service;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.core.base.entity.ConfigOilFieldEnt;
import pps.core.base.mapper.ConfigOilFieldMapper;
import pps.core.base.service.data.config_oil_field.GetConfigOilFieldOutput;
import pps.core.base.service.data.config_oil_field.GetProjectTitleOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;

import java.util.List;

/**
 * 油田配置模块
 *
 * @author ZWT
 * @date 2024/06/25
 */
@XService
public class ConfigOilFieldService {

    private static final String storeDir = XStartup.getCurrent().getProperty("home.image.path");

    /**
     * 获取当前配置
     *
     * @param context 上下文
     * @return {@link XSingleResult }<{@link GetConfigOilFieldOutput }>
     */
    @XText("获取当前配置")
    @XApiGet(anonymous = true)
    public XSingleResult<GetConfigOilFieldOutput> getCurrentConfig(XContext context) {
        ConfigOilFieldEnt ent = this.getCurrentConfigOil(context);
        GetProjectTitleOutput output = this.getProjectTitleOutput(ent.getProjectTitle());
        return XSingleResult.success(GetConfigOilFieldOutput.builder()
                .title(output.getTitle())
                .homePagePath(ent.getHomePagePath())
                .build());
    }

    /**
     * 首页图片下载
     *
     * @param context 上下文
     * @return {@link XFileResult }
     */
    @XText("首页图片下载")
    @XApiGet(anonymous = true)
    public XFileResult homeImageLoad(XContext context) {
        ConfigOilFieldEnt ent = this.getCurrentConfigOil(context);
        GetProjectTitleOutput output = this.getProjectTitleOutput(ent.getProjectTitle());
        return XFileResult.success(XFileType.IMAGE_PNG, storeDir + FileUtil.FILE_SEPARATOR + output.getImageRelativePath());
    }

    /*--------------------------------- private ---------------------------------*/

    /**
     * 获取当前配置
     *
     * @param context 上下文
     * @return {@link ConfigOilFieldEnt }
     */
    private ConfigOilFieldEnt getCurrentConfigOil(XContext context) {
        ConfigOilFieldMapper mapper = context.getBean(ConfigOilFieldMapper.class);
        return mapper.selectOne(new LambdaQueryWrapper<ConfigOilFieldEnt>()
                .eq(ConfigOilFieldEnt::getIsEnable, BusinessConstant.ZERO)
                .last(BaseUtils.getLastLimit())
        );
    }

    /**
     * 获取项目标题输出
     *
     * @param projectTitle 项目名称
     * @return {@link GetProjectTitleOutput }
     */
    private GetProjectTitleOutput getProjectTitleOutput(String projectTitle) {
        List<GetProjectTitleOutput> list = JSONUtil.toList(projectTitle, GetProjectTitleOutput.class);
        return list.stream()
                .filter(t -> ObjectUtil.isNotNull(t.getSort()) && ObjectUtil.equals(0, t.getSort()))
                .findAny()
                .orElse(new GetProjectTitleOutput());
    }
}