package pps.core.base.service.data.third_weather_data;

import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.fastjson2.annotation.JSONField;
import lombok.Data;
import xstartup.annotation.XText;

import java.math.BigDecimal;

/**
 * 第三方气象数据
 *
 * @author ZWT
 * @date 2024/04/23
 */
@Data
@ExcelIgnoreUnannotated
public class ThirdApiWeatherExcelData {

    @ExcelProperty(value = "dni")
    @XText("太阳直接辐射[W/m2]")
    private BigDecimal dni;

    @ExcelProperty(value = "ghi")
    @XText("太阳总辐射（水平面）[W/m2]")
    private BigDecimal ghi;

    @ExcelProperty(value = "dhi")
    @XText("太阳散射辐射（水平面）[W/m2]")
    private BigDecimal dhi;

    @ExcelProperty(value = "time")
    @XText("预报时间")
    @JSONField(name = "time")
    private String time;

    @ExcelProperty(value = "tem")
    @XText("气温 [C]")
    private BigDecimal tem;

    @ExcelProperty(value = "rhu")
    @XText("相对湿度 [%]")
    private BigDecimal rhu;

    @ExcelProperty(value = "wns")
    @XText("风速（地面10米）[m/s]")
    private BigDecimal wns;

    @ExcelProperty(value = "wnd")
    @XText("风向角度（地面10米）[degree]")
    private Integer wnd;

    @ExcelProperty(value = "wns_grd")
    @XText("风力等级（地面10米）")
    @JSONField(name = "wns_grd")
    private Integer wnsGrd;

    @ExcelProperty(value = "gust")
    @XText("阵风风速（地面10米）[m/s]")
    private BigDecimal gust;

    @ExcelProperty(value = "pre")
    @XText("降水量（1小时累计）[mm]")
    private BigDecimal pre;

    @ExcelProperty(value = "prs_qfe")
    @XText("地面气压 [hPa]")
    @JSONField(name = "prs_qfe")
    private BigDecimal prsQfe;

    @ExcelProperty(value = "vis")
    @XText("能见度 [km]")
    private BigDecimal vis;

    @ExcelProperty(value = "clo")
    @XText("云量 [%]")
    private BigDecimal clo;

    @ExcelProperty(value = "uvb")
    @XText("紫外线UV辐照度（1小时累计）[kWh/m2]")
    private BigDecimal uvb;

    @ExcelProperty(value = "uvi")
    @XText("紫外线UV辐照指数")
    private Integer uvi;

    @ExcelProperty(value = "phs")
    @XText("降水相态：(0无降水, 1雨, 3冻雨, 5雪, 6湿雪, 7雨夹雪, 8冰雹)")
    private Integer phs;

    @ExcelProperty(value = "wep")
    @XText("天气现象代码")
    private Integer wep;

    @ExcelProperty(value = "wnd_100m")
    @XText("风向角度（地面100米）[degree]")
    @JSONField(name = "wnd_100m")
    private BigDecimal wnd100m;

    @ExcelProperty(value = "wns_100m")
    @XText("风速（地面100米）[m/s]")
    @JSONField(name = "wns_100m")
    private BigDecimal wns100m;

    @ExcelProperty(value = "wns_grd_100m")
    @XText("风力等级（地面100米）")
    @JSONField(name = "wns_grd_100m")
    private Integer wnsGrd100m;
}
