package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;

/**
 * 日用电趋势定时任务
 *
 * @author ZWT
 * @date 2024/05/22 17:12
 */
@XText("日用电趋势定时任务")
@XService
public class DailyElectricityTrendJob {

    /**
     * 每天0点12执行(跑前一天数据)     0 12 0 * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ DailyElectricityTrendJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "DailyElectricityTrendJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IDailyElectricityTrendCloudService service = xContext.getBean(IDailyElectricityTrendCloudService.class);
                XServiceResult result = service.dailyElectricityTrendJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ DailyElectricityTrendJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ DailyElectricityTrendJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}