package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceCalibrationCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 定期校准定时任务
 *
 * @author ZWT
 * @date 2023/09/20 09:20
 */
@XText("定期校准定时任务")
@XService
public class SpaceCalibrationJob implements XJob {

    /**
     * 每天00:25执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_25)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceCalibrationJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceCalibrationJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                ISpaceCalibrationCloudService service = xContext.getBean(ISpaceCalibrationCloudService.class);
                XServiceResult result = service.calibrateJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ SpaceCalibrationJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceCalibrationJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}