package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceOptimizeLongCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 中期间开优化定时任务
 *
 * @author ZWT
 * @date 2023/09/13 13:59
 */
@XText("中期间开优化定时任务")
@XService
public class SpaceOptimizeLongJob implements XJob {

    /**
     * 每天00:15执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_15)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceOptimizeLongJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceOptimizeLongJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                ISpaceOptimizeLongCloudService service = xContext.getBean(ISpaceOptimizeLongCloudService.class);
                XServiceResult result = service.optimizeLongJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ SpaceOptimizeLongJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceOptimizeLongJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}