package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceOptimizeUltraCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 长期间开优化定时任务
 *
 * @author ZWT
 * @date 2024/03/22
 */
@XText("长期间开优化定时任务")
@XService
public class SpaceOptimizeUltraJob implements XJob {

    /**
     * 每天00:05执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_5)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceOptimizeUltraJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceOptimizeUltraJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                ISpaceOptimizeUltraCloudService service = xContext.getBean(ISpaceOptimizeUltraCloudService.class);
                XServiceResult result = service.optimizeUltraJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ SpaceOptimizeUltraJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceOptimizeUltraJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}