package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.space.service.ISpaceYesterdayGreenRateCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

/**
 * 昨日绿电占比定时任务
 *
 * @author ZWT
 * @date 2024/11/27 15:12
 */
@XText("长期间开优化定时任务")
@XService
public class SpaceYesterdayGreenRateJob implements XJob {

    /**
     * 每天00:30执行
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    @XCronTrigger(value = TaskConstant.EVERY_DAY_30)
    @Override
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ SpaceYesterdayGreenRateJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "SpaceYesterdayGreenRateJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                xContext.getBean(ISpaceYesterdayGreenRateCloudService.class)
                        .yesterdayGreenRateJob(xContext)
                        .throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ SpaceYesterdayGreenRateJob XServiceException: ", e);
            } finally {
                xContext.getLogger().info("------ SpaceYesterdayGreenRateJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
