package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IStationDailyCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;

/**
 * 场站纬度每日信息统计定时任务
 *
 * @author ZWT
 * @date 2024/10/20 16:22
 */
@XText("场站纬度每日信息统计定时任务")
public class StationDailyProductionSituationJob {

    /**
     * 每天8点40分执行   0 40 8 * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ StationDailyProductionSituationJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "StationDailyProductionSituationJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IStationDailyCloudService service = xContext.getBean(IStationDailyCloudService.class);
                XServiceResult result = service.stationDailyJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ StationDailyProductionSituationJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ StationDailyProductionSituationJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
