package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;

/**
 * 第三方当日油井状态定时任务
 *
 * @author ZWT
 * @date 2024/03/19 10:07
 */
@XText("第三方当日油井状态定时任务")
public class ThirdCurrentWellConditionJob {

    /**
     * 长庆:10分钟一次,隔9分钟   0 9/10 * * * ?
     * 吉林:30分钟一次,隔9分钟   0 9/30 * * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult}
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ ThirdCurrentWellConditionJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "ThirdCurrentWellConditionJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IThirdDataAccessCloudService service = xContext.getBean(IThirdDataAccessCloudService.class);
                XServiceResult result = service.thirdCurrentWellConditionJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ ThirdCurrentWellConditionJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ ThirdCurrentWellConditionJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}