package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IThirdWindPowerGenerationCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;

/**
 * 第三方风力发电数据定时任务
 * todo 测试用
 *
 * @author ZWT
 * @date 2024/09/03 16:22
 */
@XText("第三方风力发电数据定时任务")
@XService
public class ThirdWindPowerGenerationJob {

    /**
     * 每半小时隔1分钟执行   0 1/30 * * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult }
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ ThirdWindPowerGenerationJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "ThirdWindPowerGenerationJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IThirdWindPowerGenerationCloudService service = xContext.getBean(IThirdWindPowerGenerationCloudService.class);
                XServiceResult result = service.thirdWindPowerGenerationJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ ThirdWindPowerGenerationJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ ThirdWindPowerGenerationJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}