package pps.core.task.job;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.IWindPredictionHistoryCloudService;
import pps.core.common.cache.TaskLockCache;
import pps.core.task.constant.TaskConstant;
import pps.core.task.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XServiceResult;

/**
 * 风资源训练集数据定时任务
 *
 * @author ZWT
 * @date 2024/08/30 13:50
 */
@XText("风资源训练集数据定时任务")
@XService
public class WindPredictionHistoryJob {

    /**
     * 每天00:12执行    0 12 0 * * ?
     *
     * @param xContext x上下文
     * @return {@link XServiceResult }
     */
    public XServiceResult execute(XContext xContext) {
        xContext.getLogger().info("------ WindPredictionHistoryJob start:{}", DateUtil.date());
        String key = TaskConstant.TASK_LOCK_KEY + "WindPredictionHistoryJob";
        TaskLockCache exist = TaskLockCache.exist(xContext, key);
        if (ObjectUtil.isNull(exist)) {
            ServiceUtil.setCache(xContext, key);
            try {
                IWindPredictionHistoryCloudService service = xContext.getBean(IWindPredictionHistoryCloudService.class);
                XServiceResult result = service.windPredictionHistoryJob(xContext);
                result.throwIfFail();
            } catch (XServiceException e) {
                xContext.getLogger().error("------ WindPredictionHistoryJob Exception: ", e);
            } finally {
                xContext.getLogger().info("------ WindPredictionHistoryJob end:{}", DateUtil.date());
                TaskLockCache.delete(xContext, key);
            }
        }
        return XServiceResult.OK;
    }
}
