package pps.core.space.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import xstartup.annotation.XText;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 井口日用电趋势
 *
 * @author ZWT
 * @date 2024/05/24
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class WellDailyElectricityTrendView implements Serializable {
    @XText("ID")
    @TableField
    private String id;

    @XText("井号")
    @TableField
    private String wellNumber;

    @XText("间开每日运行时长(h)")
    @TableField
    private BigDecimal spaceRunDuration;

    @XText("每日开井时间(h)")
    @TableField
    private BigDecimal dailyOpenHour;

    @XText("绿电开井时间(h)")
    @TableField
    private BigDecimal dailyGreenOpenHour;

    @XText("日绿电用电量(kW-h)")
    @TableField
    private BigDecimal dailyGreenConsumption;

    @XText("日耗电量(kW-h)")
    @TableField
    private BigDecimal dailyElectricityConsumption;

    @XText("日产液量(m³)")
    @TableField
    private BigDecimal dailyLiquidProduction;

    @XText("日减碳量(吨)")
    @TableField
    private BigDecimal dailyCarbonReduction;

    @XText("绿电占比(%)")
    @TableField
    private BigDecimal greenElectricityRate;

    @XText("谷电开井时间(h)")
    @TableField
    private BigDecimal valleyElectricOpenHour;

    @XText("谷电占比(%)")
    @TableField
    private BigDecimal valleyElectricRate;

    @XText("创建时间")
    @TableField
    private Date createDate;

    @XText("来源系统")
    @TableField
    private String systemSource;

    /*---------------------------- other ----------------------------*/

    /**
     * 月份
     */
    @TableField(exist = false)
    private Integer month;

    /**
     * 年份
     */
    @TableField(exist = false)
    private Integer year;

    /**
     * 井号列表
     */
    @TableField(exist = false)
    private List<String> wellNumberList;

    /**
     * 月间开运行时长(h)
     */
    @TableField(exist = false)
    private BigDecimal monthSpaceRunDuration;

    /**
     * 月绿电开井时间(h)
     */
    @TableField(exist = false)
    private BigDecimal monthDailyGreenOpenHour;

    /**
     * 月绿电用电量(kW-h)
     */
    @TableField(exist = false)
    private BigDecimal monthDailyGreenConsumption;

    /**
     * 月谷电开井时间(h)
     */
    @TableField(exist = false)
    private BigDecimal monthValleyElectricOpenHour;

    /**
     * 年间开运行时长(h)
     */
    @TableField(exist = false)
    private BigDecimal yearSpaceRunDuration;

    /**
     * 年绿电开井时间(h)
     */
    @TableField(exist = false)
    private BigDecimal yearDailyGreenOpenHour;

    /**
     * 年绿电用电量(kW-h)
     */
    @TableField(exist = false)
    private BigDecimal yearDailyGreenConsumption;

    /**
     * 年谷电开井时间(h)
     */
    @TableField(exist = false)
    private BigDecimal yearValleyElectricOpenHour;
}
