package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetMonthlyWellCountInput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetMonthlyWellCountOutput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadInput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.*;
import pps.core.space.mapper.SpaceOptimizeEffectMapper;
import pps.core.space.mapper.SpaceOptimizeShortDurationMapper;
import pps.core.space.mapper.SpaceOptimizeShortWellheadMapper;
import pps.core.space.mapper.SpaceOptimizeViewMapper;
import pps.core.space.service.data.SpaceOptimizeDateDuration;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 极短期间开优化Cloud模块
 *
 * @author ZWT
 * @date 2023/09/20 11:44
 */
@XService
public class SpaceOptimizeShortCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceOptimizeShortCloudService {

    /**
     * 极短期间开优化Cloud模块--定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult optimizeShortJob(XContext context) {
        List<SpaceOptimizePeriodDTO> periodDTOList = new ArrayList<>(32);
        List<SpaceOptimizeWellheadDTO> wellheadDTOList = new ArrayList<>(64);
        List<SpaceOptimizeDurationDTO> durationDTOList = new ArrayList<>(128);
        List<SpaceOptimizeDurationDTO> unOptimizeDurationList = new ArrayList<>(128);
        super.dynamicOptimize4Api(context, periodDTOList, wellheadDTOList, durationDTOList, unOptimizeDurationList, BusinessConstant.ONE, BusinessConstant.THREE, 1);
        //开启事务
        XServiceResult result = XTransactionHelper.begin(context, () -> {
            int size;
            SpaceOptimizeViewMapper optimizeViewMapper = context.getBean(SpaceOptimizeViewMapper.class);
            if (CollUtil.isNotEmpty(periodDTOList)) {
                size = periodDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizePeriodDTO>> subList = BaseUtils.getSubList(periodDTOList);
                    subList.forEach(optimizeViewMapper::shortPeriodBatchInsertList);
                } else {
                    optimizeViewMapper.shortPeriodBatchInsertList(periodDTOList);
                }
            }
            if (CollUtil.isNotEmpty(wellheadDTOList)) {
                size = wellheadDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeWellheadDTO>> subList = BaseUtils.getSubList(wellheadDTOList);
                    subList.forEach(optimizeViewMapper::shortWellheadBatchInsertList);
                } else {
                    optimizeViewMapper.shortWellheadBatchInsertList(wellheadDTOList);
                }
            }
            if (CollUtil.isNotEmpty(unOptimizeDurationList)) {
                size = unOptimizeDurationList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(unOptimizeDurationList);
                    subList.forEach(optimizeViewMapper::shortDurationBatchInsertList);
                } else {
                    optimizeViewMapper.shortDurationBatchInsertList(unOptimizeDurationList);
                }
            }
            if (CollUtil.isNotEmpty(durationDTOList)) {
                size = durationDTOList.size();
                if (size > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceOptimizeDurationDTO>> subList = BaseUtils.getSubList(durationDTOList);
                    subList.forEach(optimizeViewMapper::shortDurationBatchInsertList);
                } else {
                    optimizeViewMapper.shortDurationBatchInsertList(durationDTOList);
                }
            }
            return XServiceResult.OK;
        });
        //计算优化效果
        super.calculateOptimizeEffect(context, periodDTOList, wellheadDTOList, durationDTOList, unOptimizeDurationList, 1, BusinessConstant.ONE);
        return result;
    }

    /**
     * 月度井口统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetMonthlyWellCountOutput }>
     */
    @Override
    public XSingleResult<GetMonthlyWellCountOutput> monthlyWellCount(XContext context, GetMonthlyWellCountInput input) {
        QueryWrapper<SpaceOptimizeShortWellheadEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("DISTINCT WELL_NUMBER")
                    .apply("EXTRACT( YEAR FROM OPTIMIZE_DATE ) = {0}", input.getMonth());
        } else {
            queryWrapper.select("DISTINCT WELL_NUMBER")
                    .apply("MONTH ( OPTIMIZE_DATE ) = {0}", input.getMonth());
        }
        SpaceOptimizeShortWellheadMapper mapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        Long count = mapper.selectCount(queryWrapper);
        return XSingleResult.success(GetMonthlyWellCountOutput.builder()
                .wellNumber(count)
                .build());
    }

    /**
     * 每日优化结果
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetSpaceOptimizeShortWellheadOutput }>
     */
    @Override
    public XListResult<GetSpaceOptimizeShortWellheadOutput> dailyOptimizationResult(XContext context, GetSpaceOptimizeShortWellheadInput input) {
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        //查间开优化井口
        SpaceOptimizeShortWellheadMapper wellheadMapper = context.getBean(SpaceOptimizeShortWellheadMapper.class);
        List<SpaceOptimizeShortWellheadEnt> wellheadList = wellheadMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortWellheadEnt>()
                .in(SpaceOptimizeShortWellheadEnt::getWellheadId, input.getWellheadIds())
                .eq(SpaceOptimizeShortWellheadEnt::getOptimizeDate, today)
        );
        if (CollUtil.isEmpty(wellheadList)) {
            return XListResult.success(Collections.emptyList());
        }
        SpaceOptimizeShortDurationMapper durationMapper = context.getBean(SpaceOptimizeShortDurationMapper.class);
        List<SpaceOptimizeShortDurationEnt> durationList;
        //查能耗
        SpaceOptimizeEffectMapper mapper = context.getBean(SpaceOptimizeEffectMapper.class);
        List<SpaceOptimizeEffectEnt> effectList = mapper.selectList(new LambdaQueryWrapper<SpaceOptimizeEffectEnt>()
                .select(SpaceOptimizeEffectEnt::getWellheadId, SpaceOptimizeEffectEnt::getGreenPowerRate, SpaceOptimizeEffectEnt::getPowerCost,
                        SpaceOptimizeEffectEnt::getOptimalGreenPowerRate, SpaceOptimizeEffectEnt::getOptimalPowerCost, SpaceOptimizeEffectEnt::getInstitutionResult,
                        SpaceOptimizeEffectEnt::getGreenPowerUpgrade, SpaceOptimizeEffectEnt::getCostDown, SpaceOptimizeEffectEnt::getOptimalResult)
                .eq(BaseModel::getIsDeleted, 1)
                .in(SpaceOptimizeEffectEnt::getPeriodId, wellheadList.stream()
                        .map(SpaceOptimizeShortWellheadEnt::getShortPeriodId)
                        .collect(Collectors.toSet()))
                .in(SpaceOptimizeEffectEnt::getWellheadId, input.getWellheadIds())
                .eq(SpaceOptimizeEffectEnt::getOptimizeDate, today)
        );
        if (CollUtil.isNotEmpty(effectList)) {
            Map<Boolean, List<SpaceOptimizeEffectEnt>> collect = effectList.stream()
                    .collect(Collectors.partitioningBy(
                            w ->
                                    /* 降低费用大于等于0才优化 */
                                    ObjectUtil.defaultIfNull(w.getCostDown(), BigDecimal.ZERO).compareTo(BigDecimal.ZERO) >= 0
                    ));
            durationList = new ArrayList<>(effectList.size());
            List<String> trueList;
            List<SpaceOptimizeShortDurationEnt> list;
            //可优化的
            List<SpaceOptimizeEffectEnt> effectEntList = collect.get(true);
            if (CollUtil.isNotEmpty(effectEntList)) {
                trueList = effectEntList.stream()
                        .map(SpaceOptimizeEffectEnt::getWellheadId)
                        .collect(Collectors.toList());
                list = this.getDurationList(durationMapper, BusinessConstant.ZERO, trueList, today);
                if (CollUtil.isNotEmpty(list)) {
                    CollUtil.addAll(durationList, list);
                }
            }
            //不可优化
            effectEntList = collect.get(false);
            if (CollUtil.isNotEmpty(effectEntList)) {
                trueList = effectEntList.stream()
                        .map(SpaceOptimizeEffectEnt::getWellheadId)
                        .collect(Collectors.toList());
                list = this.getDurationList(durationMapper, BusinessConstant.ONE, trueList, today);
                if (CollUtil.isNotEmpty(list)) {
                    CollUtil.addAll(durationList, list);
                }
            }
        } else {
            durationList = this.getDurationList(durationMapper, BusinessConstant.ZERO, input.getWellheadIds(), today);
        }
        List<pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput> wellheadOutputs = XCopyUtils.copyNewList(wellheadList, pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput.class);
        List<SpaceOptimizeDateDuration> durationOutputs = XCopyUtils.copyNewList(durationList, SpaceOptimizeDateDuration.class);
        Map<String, List<SpaceOptimizeDateDuration>> collect = durationOutputs.stream()
                .collect(Collectors.groupingBy(SpaceOptimizeDateDuration::getWellheadId));
        for (pps.core.space.service.data.space_optimize_short_wellhead.GetSpaceOptimizeShortWellheadOutput wellhead : wellheadOutputs) {
            wellhead.setOptimizeList(ServiceUtil.getAllDateDuration(collect.get(wellhead.getWellheadId())));
        }
        return XListResult.success(XCopyUtils.copyNewList(wellheadOutputs, GetSpaceOptimizeShortWellheadOutput.class));
    }

    /**
     * 获取持续时间列表
     *
     * @param durationMapper 持续时间映射器
     * @param isOptimize     已优化
     * @param wellheadIds    井口ID
     * @param today          今天
     * @return {@link List }<{@link SpaceOptimizeShortDurationEnt }>
     */
    private List<SpaceOptimizeShortDurationEnt> getDurationList(SpaceOptimizeShortDurationMapper durationMapper, Integer isOptimize, List<String> wellheadIds, DateTime today) {
        return durationMapper.selectList(new LambdaQueryWrapper<SpaceOptimizeShortDurationEnt>()
                .eq(BaseModel::getIsDeleted, BusinessConstant.ONE)
                .eq(SpaceOptimizeShortDurationEnt::getIsOptimize, isOptimize)
                .in(SpaceOptimizeShortDurationEnt::getWellheadId, wellheadIds)
                .eq(SpaceOptimizeShortDurationEnt::getOptimizeDate, today)
                .orderByAsc(SpaceOptimizeShortDurationEnt::getIsOptimize, SpaceOptimizeShortDurationEnt::getOpenWellTime)
        );
    }
}