package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailInput;
import pps.cloud.base.service.data.base_price_strategy_detail.GetBasePriceStrategyDetailOutput;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewOutput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.space.service.ISpaceYesterdayGreenRateCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.entity.SpaceYesterdayGreenRateView;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import pps.core.space.mapper.SpaceYesterdayGreenRateViewMapper;
import pps.core.space.service.data.SpaceOptimizeWeight;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 昨日绿电占比Cloud模块
 *
 * @author ZWT
 * @date 2024/11/27 15:17
 */
@XService
public class SpaceYesterdayGreenRateCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceYesterdayGreenRateCloudService {

    /**
     * 昨日绿电占比
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult yesterdayGreenRateJob(XContext context) {
        //取昨日
        DateTime beginOfYesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //取昨日间开优化井
        SpaceOptimizeShortDurationViewMapper spaceMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationViewList = spaceMapper.selectResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(beginOfYesterday)
                .build());
        Map<String, Map<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>>> durationCollect;
        Map<String, Map<Date, BigDecimal>> activeMap;
        Map<String, Map<Date, BigDecimal>> predictionMap;
        if (CollUtil.isEmpty(durationViewList)) {
            durationCollect = Collections.emptyMap();
            activeMap = Collections.emptyMap();
            predictionMap = Collections.emptyMap();
        } else {
            //间开区间Map
            durationCollect = durationViewList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName,
                                    Collectors.groupingBy(SpaceOptimizeShortDurationView::getStartSeq,
                                            Collectors.groupingBy(SpaceOptimizeShortDurationView::getIsOptimize))
                            )
                    );
            //取昨日实际发电功率
            IThirdPowerCloudService cloudService = context.getBean(IThirdPowerCloudService.class);
            XListResult<GetThirdActivePowerOutput> result = cloudService.queryPhotovoltaicPower30(context, GetThirdActivePowerInput.builder()
                    .stationNames(durationCollect.keySet())
                    .createDate(beginOfYesterday)
                    .build());
            result.throwIfFail();
            List<GetThirdActivePowerOutput> powerList = result.getResult();
            if (CollUtil.isEmpty(powerList)) {
                activeMap = Collections.emptyMap();
            } else {
                activeMap = powerList.stream()
                        .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName,
                                Collectors.toMap(GetThirdActivePowerOutput::getCreateDate, GetThirdActivePowerOutput::getPhotovoltaicPower)));
            }
            //昨日预测发电功率
            IPlantPredictedPowerCloudService service = context.getBean(IPlantPredictedPowerCloudService.class);
            XListResult<GetPredictedPowerViewOutput> predictionResult = service.queryPredictionPower30(context, GetPredictedPowerViewInput.builder()
                    .stationNames(durationCollect.keySet())
                    .dataDate(beginOfYesterday)
                    .build());
            predictionResult.throwIfFail();
            List<GetPredictedPowerViewOutput> predictionList = predictionResult.getResult();
            if (CollUtil.isEmpty(predictionList)) {
                predictionMap = Collections.emptyMap();
            } else {
                predictionMap = predictionList.stream()
                        .collect(Collectors.groupingBy(GetPredictedPowerViewOutput::getStationName,
                                Collectors.toMap(GetPredictedPowerViewOutput::getCreateTime, GetPredictedPowerViewOutput::getPredictPower)));
            }
        }
        //查市电峰谷
        List<GetBasePriceStrategyDetailOutput> strategyList = ServiceUtil.getStrategyDetailList(context, GetBasePriceStrategyDetailInput.builder()
                .strategyMonth(String.valueOf(beginOfYesterday.monthBaseOne()))
                .build());
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);
        List<SpaceOptimizeShortDurationView> unOptimizeList;
        List<SpaceOptimizeShortDurationView> optimizeList;
        TreeMap<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>> wellSortMap;
        SpaceOptimizeShortDurationView durationTemp;
        List<SpaceYesterdayGreenRateView> batchList = new ArrayList<>(durationCollect.size() * 2);
        SpaceOptimizeShortDurationView unOptimizeDuration;
        SpaceOptimizeShortDurationView optimizeDuration;
        DateTime openWellTime;
        DateTime closeWellTime;
        for (Map.Entry<String, Map<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>>> lineEntry : durationCollect.entrySet()) {
            String lineName = lineEntry.getKey();
            //初始化轴
            Map<DateTime, SpaceOptimizeWeight> activeWeightMap = this.obtainTimeline(strategyList, rangeToList, activeMap.getOrDefault(lineName, Collections.emptyMap()));
            Map<DateTime, SpaceOptimizeWeight> predictionWeightMap = this.obtainTimeline(strategyList, rangeToList, predictionMap.getOrDefault(lineName, Collections.emptyMap()));
            //按顺序排
            wellSortMap = new TreeMap<>(lineEntry.getValue());
            BigDecimal sumServiceRating = BigDecimal.ZERO;
            //计算结果
            for (Map.Entry<Integer, Map<Integer, List<SpaceOptimizeShortDurationView>>> sortEntry : wellSortMap.entrySet()) {
                //未优化间开
                unOptimizeList = sortEntry.getValue().get(1);
                if (CollUtil.isEmpty(unOptimizeList)) {
                    continue;
                }
                BigDecimal wellGreenPowerActual = BigDecimal.ZERO;
                BigDecimal wellGreenPowerOptimize = BigDecimal.ZERO;
                BigDecimal runHourActual = BigDecimal.ZERO;
                BigDecimal runHourOptimize = BigDecimal.ZERO;
                BigDecimal powerCostActual = BigDecimal.ZERO;
                BigDecimal powerCostOptimize = BigDecimal.ZERO;
                BigDecimal cityCostActual = BigDecimal.ZERO;
                BigDecimal cityCostOptimize = BigDecimal.ZERO;
                durationTemp = unOptimizeList.get(0);
                //累加运行功率
                BigDecimal serviceRating = ObjectUtil.defaultIfNull(durationTemp.getServiceRating(), BigDecimal.ZERO);
                sumServiceRating = sumServiceRating.add(serviceRating);
                //优化后间开
                optimizeList = sortEntry.getValue().get(0);
                boolean optimizeListFlag = CollUtil.isNotEmpty(optimizeList);
                int unOptimizeIndex = 0;
                int optimizeIndex = 0;
                boolean greenFlag = false;
                //计算优化前后功率
                for (DateTime dateTime : rangeToList) {
                    BigDecimal greenPowerActual = BigDecimal.ZERO;
                    BigDecimal greenPowerPrediction = BigDecimal.ZERO;
                    //计算实际
                    SpaceOptimizeWeight optimizeWeight = activeWeightMap.get(dateTime);
                    BigDecimal activePower = optimizeWeight.getPower();
                    unOptimizeDuration = unOptimizeList.get(unOptimizeIndex);
                    //时间转换
                    openWellTime = this.parseDateTime(unOptimizeDuration.getOpenWellTime());
                    closeWellTime = this.parseDateTime(unOptimizeDuration.getCloseWellTime());
                    if (DateUtil.compare(dateTime, openWellTime) >= 0 && DateUtil.compare(dateTime, closeWellTime) < 0) {
                        //计算发电功率
                        if (activePower.compareTo(BigDecimal.ZERO) <= 0) {
                            //不管
                        } else if (activePower.compareTo(serviceRating) >= 0) {
                            greenFlag = true;
                            //光伏发电功率大于运行负荷: 该时段光伏用电量量=运行负荷*0.5
                            greenPowerActual = serviceRating.multiply(BusinessConstant.BIG_DECIMAL_0_5);
                            wellGreenPowerActual = wellGreenPowerActual.add(greenPowerActual);
                            activeWeightMap.get(dateTime).setPower(activePower.subtract(serviceRating));
                        } else {
                            greenFlag = true;
                            //光伏发电功率-井口累计发电功率 : 小于等于0不管
                            BigDecimal subtract = activePower.subtract(serviceRating);
                            if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                                greenPowerActual = subtract;
                                //光伏发电功率小于井的运行负荷: 该时段光伏用电量=光伏发电功率*0.5
                                wellGreenPowerActual = wellGreenPowerActual.add(subtract.multiply(BusinessConstant.BIG_DECIMAL_0_5));
                            }
                            activeWeightMap.get(dateTime).setPower(BigDecimal.ZERO);
                        }
                        //累加开井时长
                        runHourActual = runHourActual.add(BusinessConstant.BIG_DECIMAL_0_5);
                        //计算电费(绿电没电费)
                        if (greenFlag) {
                            //绿电市电电费
                            cityCostActual = cityCostActual.add(optimizeWeight.getElectricityPrice().multiply(greenPowerActual));
                        } else {
                            //市电峰谷电费
                            powerCostActual = powerCostActual.add(optimizeWeight.getElectricityPrice().multiply(BusinessConstant.BIG_DECIMAL_0_5));
                        }
                    } else if (DateUtil.compare(dateTime, closeWellTime) >= 0) {
                        unOptimizeIndex++;
                    }
                    if (unOptimizeIndex == unOptimizeList.size()) {
                        unOptimizeIndex = unOptimizeList.size() - 1;
                    }
                    /*--------------------------------------------------*/
                    greenFlag = false;
                    if (optimizeListFlag) {
                        SpaceOptimizeWeight predictionWeight = predictionWeightMap.get(dateTime);
                        BigDecimal predictionPower = predictionWeight.getPower();
                        optimizeDuration = optimizeList.get(optimizeIndex);
                        //时间转换
                        openWellTime = this.parseDateTime(optimizeDuration.getOpenWellTime());
                        closeWellTime = this.parseDateTime(optimizeDuration.getCloseWellTime());
                        if (DateUtil.compare(dateTime, openWellTime) >= 0 && DateUtil.compare(dateTime, closeWellTime) < 0) {
                            //计算发电功率
                            if (predictionPower.compareTo(BigDecimal.ZERO) <= 0) {
                                //不管
                            } else if (predictionPower.compareTo(sumServiceRating) >= 0) {
                                greenFlag = true;
                                //光伏发电功率大于运行负荷: 该时段光伏用电量量=运行负荷*0.5
                                greenPowerPrediction = serviceRating.multiply(BusinessConstant.BIG_DECIMAL_0_5);
                                wellGreenPowerOptimize = wellGreenPowerOptimize.add(greenPowerPrediction);
                                predictionWeightMap.get(dateTime).setPower(predictionPower.subtract(serviceRating));
                            } else {
                                greenFlag = true;
                                //光伏发电功率-井口累计发电功率 : 小于等于0不管
                                BigDecimal subtract = predictionPower.subtract(serviceRating);
                                if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                                    greenPowerPrediction = subtract;
                                    //光伏发电功率小于井的运行负荷: 该时段光伏用电量=(光伏发电功率-井口累计发电功率+当前井口运行功率)*0.5
                                    wellGreenPowerOptimize = wellGreenPowerOptimize.add(subtract.multiply(BusinessConstant.BIG_DECIMAL_0_5));
                                }
                                predictionWeightMap.get(dateTime).setPower(BigDecimal.ZERO);
                            }
                            //累加开井时长
                            runHourOptimize = runHourOptimize.add(BusinessConstant.BIG_DECIMAL_0_5);
                            //计算电费(绿电没电费)
                            if (greenFlag) {
                                //绿电市电电费
                                cityCostOptimize = cityCostOptimize.add(optimizeWeight.getElectricityPrice().multiply(greenPowerPrediction));
                            } else {
                                //市电峰谷电费
                                powerCostOptimize = powerCostOptimize.add(optimizeWeight.getElectricityPrice().multiply(BusinessConstant.BIG_DECIMAL_0_5));
                            }
                        } else if (DateUtil.compare(dateTime, closeWellTime) >= 0) {
                            optimizeIndex++;
                        }
                        if (optimizeIndex == optimizeList.size()) {
                            optimizeIndex = optimizeList.size() - 1;
                        }
                    }
                }
                //添加计算结果
                BigDecimal wellTotalPowerActual = runHourActual.multiply(serviceRating);
                BigDecimal wellTotalPowerOptimize = runHourOptimize.multiply(serviceRating);
                SpaceYesterdayGreenRateView yesterdayGreenRateView = new SpaceYesterdayGreenRateView();
                BaseUtils.setBaseModelDefaultForJob(yesterdayGreenRateView);
                yesterdayGreenRateView.setRecordDate(beginOfYesterday);
                yesterdayGreenRateView.setWellheadId(durationTemp.getWellheadId());
                yesterdayGreenRateView.setWellNumber(durationTemp.getWellNumber());
                yesterdayGreenRateView.setStartSeq(sortEntry.getKey());
                yesterdayGreenRateView.setWellTotalPowerActual(wellTotalPowerActual);
                yesterdayGreenRateView.setWellTotalPowerOptimize(wellTotalPowerOptimize);
                yesterdayGreenRateView.setWellGreenPowerActual(wellGreenPowerActual);
                yesterdayGreenRateView.setWellGreenPowerOptimize(wellGreenPowerOptimize);
                yesterdayGreenRateView.setGreenPowerRateActual(BaseUtils.getRate(wellGreenPowerActual, wellTotalPowerActual));
                yesterdayGreenRateView.setGreenPowerRateOptimize(BaseUtils.getRate(wellGreenPowerOptimize, wellTotalPowerOptimize));
                yesterdayGreenRateView.setLineName(lineName);
                yesterdayGreenRateView.setPowerCostActual(powerCostActual);
                yesterdayGreenRateView.setPowerCostOptimize(powerCostOptimize);
                yesterdayGreenRateView.setCityCostActual(cityCostActual);
                yesterdayGreenRateView.setCityCostOptimize(cityCostOptimize);
                batchList.add(yesterdayGreenRateView);
            }
        }
        return XTransactionHelper.begin(context, () -> {
            SpaceYesterdayGreenRateViewMapper mapper = context.getBean(SpaceYesterdayGreenRateViewMapper.class);
            if (CollUtil.isNotEmpty(batchList)) {
                if (batchList.size() > BaseUtils.BATCH_SIZE) {
                    List<List<SpaceYesterdayGreenRateView>> subList = BaseUtils.getSubList(batchList);
                    subList.forEach(mapper::batchInsert);
                } else {
                    mapper.batchInsert(batchList);
                }
            }
            return XServiceResult.OK;
        });
    }


    /**
     * 获取时间线
     *
     * @param strategyList   战略列表
     * @param rangeToList    要列出范围
     * @param activePowerMap 有功功率图
     * @return {@link Map }<{@link DateTime }, {@link SpaceOptimizeWeight }>
     */
    private Map<DateTime, SpaceOptimizeWeight> obtainTimeline(List<GetBasePriceStrategyDetailOutput> strategyList, List<DateTime> rangeToList, Map<Date, BigDecimal> activePowerMap) {
        Map<DateTime, SpaceOptimizeWeight> weightMap = new HashMap<>(strategyList.size());
        GetBasePriceStrategyDetailOutput strategy;
        int strategyIndex = 0;
        Date openTime;
        Date closeTime;
        SpaceOptimizeWeight weight;
        DateTime dateTime;
        for (int i = 0; i < rangeToList.size(); i++) {
            dateTime = rangeToList.get(i);
            weight = new SpaceOptimizeWeight();
            weight.setPower(activePowerMap.getOrDefault(dateTime, BigDecimal.ZERO));
            weight.setTimestamp(dateTime);
            weight.setSort(i);
            strategy = strategyList.get(strategyIndex);
            openTime = strategy.getOpenTime();
            closeTime = strategy.getCloseTime();
            if (ObjectUtil.isNull(strategy.getCloseTime()) && CharSequenceUtil.equals(strategy.getEndTime(), BusinessConstant.END_OF_DAY_TIME)) {
                closeTime = BusinessConstant.DATE_FLAG;
            }
            if (DateUtil.isIn(dateTime, openTime, closeTime)) {
                weight.setWeight(super.getWeightByPeriodTypeKey(strategy.getPeriodTypeKey()));
                weight.setElectricityPrice(strategy.getElectrovalence());
                if (DateUtil.compare(dateTime, closeTime) == 0) {
                    strategyIndex++;
                }
            } else {
                strategyIndex++;
            }
            if (strategyIndex == strategyList.size()) {
                strategyIndex = strategyList.size() - 1;
            }
            weightMap.put(dateTime, weight);
        }
        return weightMap;
    }

    /**
     * 转换时间
     *
     * @param timeString 时间字符串
     * @return {@link DateTime }
     */
    private DateTime parseDateTime(String timeString) {
        DateTime wellTime;
        if (CharSequenceUtil.equals(timeString, BusinessConstant.END_OF_DAY_TIME)) {
            wellTime = BusinessConstant.DATE_FLAG;
        } else {
            wellTime = DateUtil.parse(timeString + BusinessConstant.INITIALIZATION_SECOND, BusinessConstant.TIME_FORMAT);
        }
        return wellTime;
    }
}