package pps.cloud.prediction.service;

import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

/**
 * 光伏预测Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 18:49
 */
@XService
@XText("光伏预测Cloud模块")
public interface IPlantPredictedPowerCloudService {

    /**
     * 光伏预测Cloud模块--条件查询时段发电量平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @XText("光伏预测Cloud模块--条件查询时段发电量平均值")
    XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input);

    /**
     * 获得平均功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @XText("光伏预测Cloud模块--获得平均功率")
    XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePower(XContext context, DynamicQueryPlantPredictedPowerInput input);

    /**
     * 光伏预测Cloud模块--获取每小时平均发电量列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @XText("光伏预测Cloud模块--获取每小时平均发电量列表")
    XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationHourListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input);

    /**
     * 光伏预测Cloud模块--每30分钟一天预测发电量
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @XText("光伏预测Cloud模块--每30分钟一天预测发电量")
    XListResult<GetPredictedPowerViewOutput> queryPredictionPower30(XContext context, GetPredictedPowerViewInput input);

    /**
     * 日耗电日产液信息定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @XText("光伏预测Cloud模块--日耗电日产液信息定时任务")
    XServiceResult wellTechDailyJob(XContext context);
}