package pps.cloud.prediction.service;

import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateInput;
import pps.cloud.prediction.service.data.third_daily_accumulation_update.GetThirdDailyAccumulationUpdateOutput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.cloud.prediction.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;

/**
 * 第三方有功功率Cloud模块
 *
 * @author ZWT
 * @date 2024/05/23 14:14
 */
@XService
@XText("第三方Cloud模块")
public interface IThirdPowerCloudService {

    /**
     * 获得15分钟平均功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdActivePowerOutput }>
     */
    @XText("第三方Cloud模块--获得15分钟平均功率")
    XListResult<GetThirdActivePowerOutput> queryAvgPhotovoltaicPower(XContext context, GetThirdActivePowerInput input);

    /**
     * 获得30分钟实际功率
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdActivePowerOutput }>
     */
    @XText("第三方Cloud模块--获得15分钟平均功率")
    XListResult<GetThirdActivePowerOutput> queryPhotovoltaicPower30(XContext context, GetThirdActivePowerInput input);


    /**
     * 查询每日井场日累计数据
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetThirdDailyAccumulationUpdateOutput }>
     */
    @XText("第三方Cloud模块--查询每日井场日累计数据")
    XListResult<GetThirdDailyAccumulationUpdateOutput> queryDailyUpdatePower(XContext context, GetThirdDailyAccumulationUpdateInput input);

    /**
     * 查询每日井口生产情况
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetWellheadDailyProductionSituationOutput }>
     */
    @XText("第三方Cloud模块--查询每日井口生产情况")
    XListResult<GetWellheadDailyProductionSituationOutput> queryWellProductionList(XContext context, GetWellheadDailyProductionSituationInput input);
}
