package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import com.github.pagehelper.page.PageMethod;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantInput;
import pps.cloud.base.service.data.base_power_line_plant.DynamicQueryBasePowerLinePlantViewOutput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadInput;
import pps.cloud.base.service.data.base_power_line_wellhead.DynamicQueryBasePowerLineWellheadViewOutput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.space.service.IDailyElectricityTrendCloudService;
import pps.cloud.space.service.ISpaceInstitutionDetailCloudService;
import pps.cloud.space.service.ISpaceOptimizeShortCloudService;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendInput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetLineDailyElectricityTrendOutput;
import pps.cloud.space.service.data.line_daily_electricity_trend.GetValleyElectricRateOutput;
import pps.cloud.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadInput;
import pps.cloud.space.service.data.space_institution_wellhead.GetSpaceInstitutionWellheadOutput;
import pps.cloud.space.service.data.space_optimize_short_wellhead.*;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendInput;
import pps.cloud.space.service.data.well_daily_electricity_trend.GetWellDailyElectricityTrendOutput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.*;
import pps.core.prediction.mapper.*;
import pps.core.prediction.service.data.energy_consumption_analysis.*;
import pps.core.prediction.service.data.home_page.GetPowerAnalyseOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetRealTimeInfoOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageData;
import xstartup.data.XPageResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 能耗分析模块
 *
 * @author ZWT
 * @date 2023/11/23
 */
@XText("能耗分析模块")
@XService
public class EnergyConsumptionAnalysisService {

    /*------------------------------ 间开效果评价(吉林) ------------------------------*/

    /**
     * 场站分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult }<{@link GetWellheadAnalysisOutput }>
     */
    @XText("间开效果评价--场站分析")
    @XApiPost
    public XPageResult<GetWellheadAnalysisOutput> wellStatisticsPage(XContext context, GetStatisticsPageInput input) {
        List<GetBasePhotovoltaicPlantCloudOutput> plantList = ServiceUtil.getBasePhotovoltaicPlantList(context, GetBasePhotovoltaicPlantCloudInput.builder()
                .stationName(input.getWellNumber())
                .ouIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()))
                .build());
        if (CollUtil.isNotEmpty(plantList)) {
            //日期初始化
            input.getBetweenDate();
            Date beginTime = input.getBeginTime();
            Date endTime = input.getEndTime();
            DateTime now = DateUtil.date();
            //分页查询
            PageMethod.startPage(input.getPage(), input.getLimit());
            StationDailyProductionSituationViewMapper mapper = context.getBean(StationDailyProductionSituationViewMapper.class);
            List<StationDailyProductionSituationView> list = mapper.selectStationAnalysisList(StationDailyProductionSituationView.builder()
                    .beginTime(beginTime)
                    .endTime(endTime)
                    .stationName(input.getWellNumber())
                    .stationNameList(plantList.stream()
                            .map(GetBasePhotovoltaicPlantCloudOutput::getStationName)
                            .collect(Collectors.toSet()))
                    .month(now.monthBaseOne())
                    .year(now.year())
                    .build());
            if (CollUtil.isEmpty(list)) {
                return XPageResult.success(Collections.emptyList(), input, 0);
            }
            PageInfo<StationDailyProductionSituationView> pageInfo = new PageInfo<>(list);
            List<StationDailyProductionSituationView> items = pageInfo.getList();
            List<GetWellheadAnalysisOutput> outputs = new ArrayList<>(items.size());
            for (StationDailyProductionSituationView item : items) {
                outputs.add(GetWellheadAnalysisOutput.builder()
                        .wellNumber(item.getStationName())
                        .spaceRunDuration(item.getProductionTime())
                        .dailyLiquidProduction(item.getDailyLiquidProduction())
                        .dailyElectricityConsumption(item.getDailyElectricityConsumption())
                        .dailyCarbonReduction(ServiceUtil.calculateCarbonReduction(item.getPhotovoltaicPower()))
                        .monthDailyCarbonReduction(ServiceUtil.calculateCarbonReduction(item.getMonthPhotovoltaicPower()))
                        .yearDailyCarbonReduction(ServiceUtil.calculateCarbonReduction(item.getYearPhotovoltaicPower()))
                        .photovoltaicPower(item.getPhotovoltaicPower())
                        .build());
            }
            return XPageResult.success(outputs, input, pageInfo.getTotal());
        }
        return XPageResult.success(Collections.emptyList(), input, 0);
    }

    /**
     * 累节电能
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetEconomicBenefitsOutput }>
     */
    @XText("间开效果评价--累节电能")
    @XApiPost
    public XListResult<GetEconomicBenefitsOutput> conserveStatisticsList(XContext context, GetStatisticsInput input) {
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //查组织机构下所有线路
        Set<String> lineNameSet = this.getLineNameSet(context, input.getOuId());
        boolean yearFlag = input.getDateType().equals(3);
        //统计
        List<StationDailyProductionSituationEnt> currentList = this.getStationDailyStatisticsList(context, beginTime, endTime, lineNameSet, yearFlag);
        Map<Date, BigDecimal> currentMap;
        if (CollUtil.isEmpty(currentList)) {
            currentMap = Collections.emptyMap();
        } else {
            currentMap = currentList.stream()
                    .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, StationDailyProductionSituationEnt::getPhotovoltaicPower));
        }
        //数据封装
        DateTime now = DateUtil.date();
        List<DateTime> rangeToList = DateUtil.rangeToList(beginTime, endTime, yearFlag ? DateField.MONTH : DateField.DAY_OF_YEAR);
        List<GetEconomicBenefitsOutput> outputs = new ArrayList<>(rangeToList.size());
        BigDecimal power;
        for (DateTime dateTime : rangeToList) {
            if (DateUtil.compare(dateTime, now) > 0) {
                break;
            }
            power = currentMap.getOrDefault(dateTime, BigDecimal.ZERO);
            outputs.add(GetEconomicBenefitsOutput.builder()
                    .saveElectricity(power)
                    .electricityFees(this.calculateElectricityFees(power))
                    .dateStr(yearFlag ? dateTime.toString(BusinessConstant.DATE_FORMAT_MONTH) : dateTime.toDateStr())
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 减碳量统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetEconomicBenefitsOutput }>
     */
    @XText("间开效果评价--减碳量统计")
    @XApiPost
    public XListResult<GetEconomicBenefitsOutput> carbonStatisticsList(XContext context, GetStatisticsInput input) {
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //查组织机构下所有线路
        Set<String> lineNameSet = this.getLineNameSet(context, input.getOuId());
        boolean yearFlag = input.getDateType().equals(3);
        //统计
        List<StationDailyProductionSituationEnt> currentList = this.getStationDailyStatisticsList(context, beginTime, endTime, lineNameSet, yearFlag);
        List<StationDailyProductionSituationEnt> lastList = this.getStationDailyStatisticsList(context, DateUtil.offset(beginTime, DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), lineNameSet, yearFlag);
        Map<Date, BigDecimal> currentMap;
        if (CollUtil.isEmpty(currentList)) {
            currentMap = Collections.emptyMap();
        } else {
            currentMap = currentList.stream()
                    .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, StationDailyProductionSituationEnt::getPhotovoltaicPower));
        }
        Map<Date, BigDecimal> lastMap;
        if (CollUtil.isEmpty(lastList)) {
            lastMap = Collections.emptyMap();
        } else {
            lastMap = lastList.stream()
                    .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, StationDailyProductionSituationEnt::getPhotovoltaicPower));
        }
        //数据封装
        DateTime now = DateUtil.date();
        List<DateTime> rangeToList = DateUtil.rangeToList(beginTime, endTime, yearFlag ? DateField.MONTH : DateField.DAY_OF_YEAR);
        List<GetEconomicBenefitsOutput> outputs = new ArrayList<>(rangeToList.size());
        GetEconomicBenefitsOutput output;
        for (DateTime dateTime : rangeToList) {
            if (DateUtil.compare(dateTime, now) > 0) {
                break;
            }
            output = new GetEconomicBenefitsOutput();
            output.setCarbonReduction(ServiceUtil.calculateCarbonReduction(currentMap.getOrDefault(dateTime, BigDecimal.ZERO)));
            output.setLastCarbonReduction(ServiceUtil.calculateCarbonReduction(lastMap.getOrDefault(dateTime, BigDecimal.ZERO)));
            output.setDateStr(yearFlag ? dateTime.toString(BusinessConstant.DATE_FORMAT_MONTH) : dateTime.toDateStr());
            outputs.add(output);
        }
        return XListResult.success(outputs);
    }

    /**
     * 结果统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetMonthlyOverviewOutput }>
     */
    @XText("间开效果评价--结果统计")
    @XApiPost
    public XSingleResult<GetMonthlyOverviewOutput> resultStatistics(XContext context, GetStatisticsInput input) {
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //初始化
        GetMonthlyOverviewOutput output = GetMonthlyOverviewOutput.builder()
                .accumulatePowerGeneration(BigDecimal.ZERO)
                .dailyElectricityConsumption(BigDecimal.ZERO)
                .economicBenefit(BigDecimal.ZERO)
                .carbonReduction(BigDecimal.ZERO)
                .build();
        //查组织机构下所有线路
        Set<String> lineNameSet = this.getLineNameSet(context, input.getOuId());
        //统计电量
        StationDailyProductionSituationEnt statistics = this.getStationDailyStatistics(context, beginTime, endTime, lineNameSet);
        if (ObjectUtil.isNotNull(statistics)) {
            output.setAccumulatePowerGeneration(statistics.getPhotovoltaicPower().setScale(2, RoundingMode.HALF_UP));
            output.setDailyElectricityConsumption(statistics.getDailyElectricityConsumption().setScale(2, RoundingMode.HALF_UP));
            output.setEconomicBenefit(ServiceUtil.getEconomicBenefitYuan(statistics.getPhotovoltaicPower(), BusinessConstant.ENV_SY));
            output.setCarbonReduction(ServiceUtil.calculateCarbonReduction(statistics.getPhotovoltaicPower()));
        }
        return XSingleResult.success(output);
    }

    /*------------------------------ 能耗分析(吉林) ------------------------------*/

    /**
     * 电量统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetElectricityStatisticsOutput }>
     */
    @XText("能耗分析--电量统计")
    @XApiPost
    public XSingleResult<GetElectricityStatisticsOutput> electricityStatistics(XContext context, GetStatisticsInput input) {
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //初始化
        GetElectricityStatisticsOutput output = GetElectricityStatisticsOutput.builder()
                .currentPeriodGenerate(BigDecimal.ZERO)
                .lastPeriodGenerate(BigDecimal.ZERO)
                .currentPeriodUse(BigDecimal.ZERO)
                .lastPeriodUse(BigDecimal.ZERO)
                .build();
        //查组织机构下所有线路
        Set<String> lineNameSet = this.getLineNameSet(context, input.getOuId());
        //统计电量
        StationDailyProductionSituationEnt statistics = this.getStationDailyStatistics(context, beginTime, endTime, lineNameSet);
        output.setCurrentPeriodGenerate(statistics.getPhotovoltaicPower());
        output.setCurrentPeriodUse(statistics.getDailyElectricityConsumption());
        statistics = this.getStationDailyStatistics(context, DateUtil.offset(beginTime, DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), lineNameSet);
        output.setLastPeriodGenerate(statistics.getPhotovoltaicPower());
        output.setLastPeriodUse(statistics.getDailyElectricityConsumption());
        return XSingleResult.success(output);
    }

    /**
     * 发电量统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetElectricityStatisticsOutput }>
     */
    @XText("能耗分析--发电量统计")
    @XApiPost
    public XListResult<GetElectricityStatisticsOutput> generateStatisticsList(XContext context, GetStatisticsInput input) {
        return XListResult.success(this.getStatisticsList(context, input, false));
    }

    /**
     * 用电量统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetElectricityStatisticsOutput }>
     */
    @XText("能耗分析--用电量统计")
    @XApiPost
    public XListResult<GetElectricityStatisticsOutput> useStatisticsList(XContext context, GetStatisticsInput input) {
        return XListResult.success(this.getStatisticsList(context, input, true));
    }

    /**
     * 峰谷能耗对比分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetEnergyComparisonAnalysisOutput }>
     */
    @XText("能耗分析--峰谷能耗对比分析")
    @XApiPost
    public XListResult<GetEnergyComparisonAnalysisOutput> feeStatisticsList(XContext context, GetStatisticsInput input) {
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //查组织机构下所有线路
        Set<String> lineNameSet = this.getLineNameSet(context, input.getOuId());
        boolean yearFlag = input.getDateType().equals(3);
        //统计
        QueryWrapper<StationDailyProductionSituationEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("NVL( SUM( ELECTRICITY_CHARGE_CITY ), 0 ) AS ELECTRICITY_CHARGE_CITY",
                            "NVL( SUM( ELECTRICITY_CONSUMPTION_VH ), 0 ) AS ELECTRICITY_CONSUMPTION_VH",
                            "NVL( SUM( ELECTRICITY_CONSUMPTION_NP ), 0 ) AS ELECTRICITY_CONSUMPTION_NP",
                            "NVL( SUM( ELECTRICITY_CONSUMPTION_HH ), 0 ) AS ELECTRICITY_CONSUMPTION_HH",
                            "NVL( SUM( ELECTRICITY_CONSUMPTION_PH ), 0 ) AS ELECTRICITY_CONSUMPTION_PH",
                            yearFlag ? "TO_CHAR( DATA_DATE, 'YYYY-MM-' ) || '01 00:00:00' AS DATA_DATE" : "DATA_DATE")
                    .last(yearFlag ? "GROUP BY TO_CHAR( DATA_DATE, 'YYYY-MM-' )" : "GROUP BY data_date");
        } else {
            queryWrapper.select("IFNULL( SUM( ELECTRICITY_CHARGE_CITY ), 0 ) AS ELECTRICITY_CHARGE_CITY",
                            "IFNULL( SUM( ELECTRICITY_CONSUMPTION_VH ), 0 ) AS ELECTRICITY_CONSUMPTION_VH",
                            "IFNULL( SUM( ELECTRICITY_CONSUMPTION_NP ), 0 ) AS ELECTRICITY_CONSUMPTION_NP",
                            "IFNULL( SUM( ELECTRICITY_CONSUMPTION_HH ), 0 ) AS ELECTRICITY_CONSUMPTION_HH",
                            "IFNULL( SUM( ELECTRICITY_CONSUMPTION_PH ), 0 ) AS ELECTRICITY_CONSUMPTION_PH",
                            yearFlag ? "DATE_FORMAT( DATA_DATE, '%Y-%M-01' ) AS DATA_DATE" : "DATA_DATE")
                    .last(yearFlag ? "GROUP BY DATE_FORMAT( DATA_DATE, '%Y-%m-01')" : "GROUP BY DATA_DATE");
        }
        List<StationDailyProductionSituationEnt> list = context.getBean(StationDailyProductionSituationMapper.class)
                .selectList(queryWrapper.lambda()
                        .between(StationDailyProductionSituationEnt::getDataDate, beginTime, endTime)
                        .in(StationDailyProductionSituationEnt::getStationName, lineNameSet)
                );
        Map<Date, StationDailyProductionSituationEnt> collect;
        if (CollUtil.isEmpty(list)) {
            collect = Collections.emptyMap();
        } else {
            collect = list.stream()
                    .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, Function.identity()));
        }
        //封装数据
        DateTime now = DateUtil.date();
        List<DateTime> rangeToList = DateUtil.rangeToList(beginTime, endTime, yearFlag ? DateField.MONTH : DateField.DAY_OF_YEAR);
        List<GetEnergyComparisonAnalysisOutput> outputs = new ArrayList<>(rangeToList.size());
        StationDailyProductionSituationEnt dailyEnt;
        BigDecimal electricityFees;
        BigDecimal valleyElectricPower;
        BigDecimal ordinaryElectricPower;
        BigDecimal highElectricPower;
        BigDecimal peakElectricPower;
        for (DateTime dateTime : rangeToList) {
            if (DateUtil.compare(dateTime, now) > 0) {
                break;
            }
            if (collect.containsKey(dateTime)) {
                dailyEnt = collect.get(dateTime);
                electricityFees = dailyEnt.getElectricityChargeCity();
                valleyElectricPower = dailyEnt.getElectricityConsumptionVh();
                ordinaryElectricPower = dailyEnt.getElectricityConsumptionNp();
                highElectricPower = dailyEnt.getElectricityConsumptionHh();
                peakElectricPower = dailyEnt.getElectricityConsumptionPh();
            } else {
                electricityFees = BigDecimal.ZERO;
                valleyElectricPower = BigDecimal.ZERO;
                ordinaryElectricPower = BigDecimal.ZERO;
                highElectricPower = BigDecimal.ZERO;
                peakElectricPower = BigDecimal.ZERO;
            }
            outputs.add(GetEnergyComparisonAnalysisOutput.builder()
                    .electricityFees(electricityFees)
                    .valleyElectricPower(valleyElectricPower)
                    .ordinaryElectricPower(ordinaryElectricPower)
                    .highElectricPower(highElectricPower)
                    .peakElectricPower(peakElectricPower)
                    .dateFormat(yearFlag ? dateTime.toString(BusinessConstant.DATE_FORMAT_MONTH) : dateTime.toDateStr())
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 用能分析
     * 15天以内,按小时展示
     * 月:按日展示
     * 年:按月展示
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerAnalyseOutput }>
     */
    @XText("能耗分析--用能分析")
    @XApiPost
    public XListResult<GetPowerAnalyseOutput> energyAnalyse(XContext context, GetStatisticsInput input) {
        List<String> orgIds = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //判断时间范围
        int dayFlag;
        long day = DateUtil.betweenDay(beginTime, endTime, false);
        if (day <= 15) {
            //小时
            dayFlag = 1;
        } else if (day <= 90) {
            //天
            dayFlag = 2;
        } else {
            //月
            dayFlag = 3;
        }
        //用电功率
        Map<Date, BigDecimal> avgMap = this.queryPowerConsumptionHour(context,
                ServiceUtil.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                        .ouIds(orgIds)
                        .build()),
                beginTime,
                endTime,
                oilFieldCode,
                dayFlag
        );
        //发电功率
        Map<Date, BigDecimal> powerMap = this.queryGeneratedPowerHour(context,
                ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                        .ouIds(orgIds)
                        .build()),
                beginTime,
                endTime,
                oilFieldCode,
                dayFlag
        );
        //封装数据
        return XListResult.success(this.setPowerAnalyseList(avgMap, powerMap, beginTime, endTime, oilFieldCode, dayFlag));
    }

    /**
     * 获取统计列表
     *
     * @param context 上下文
     * @param input   输入
     * @param isUse   是否用电
     * @return {@link List }<{@link GetElectricityStatisticsOutput }>
     */
    private List<GetElectricityStatisticsOutput> getStatisticsList(XContext context, GetStatisticsInput input, boolean isUse) {
        //日期初始化
        input.getBetweenDate();
        Date beginTime = input.getBeginTime();
        Date endTime = input.getEndTime();
        //查组织机构下所有线路
        Set<String> lineNameSet = this.getLineNameSet(context, input.getOuId());
        boolean yearFlag = input.getDateType().equals(3);
        //统计
        List<StationDailyProductionSituationEnt> currentList = this.getStationDailyStatisticsList(context, beginTime, endTime, lineNameSet, yearFlag);
        Map<Date, BigDecimal> currentMap;
        if (CollUtil.isEmpty(currentList)) {
            currentMap = Collections.emptyMap();
        } else {
            currentMap = currentList.stream()
                    .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, isUse ? StationDailyProductionSituationEnt::getDailyElectricityConsumption : StationDailyProductionSituationEnt::getPhotovoltaicPower));
        }
        List<StationDailyProductionSituationEnt> lastList = this.getStationDailyStatisticsList(context, DateUtil.offset(beginTime, DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), lineNameSet, yearFlag);
        Map<Date, BigDecimal> lastMap;
        if (CollUtil.isEmpty(lastList)) {
            lastMap = Collections.emptyMap();
        } else {
            lastMap = lastList.stream()
                    .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, isUse ? StationDailyProductionSituationEnt::getDailyElectricityConsumption : StationDailyProductionSituationEnt::getPhotovoltaicPower));
        }
        //封装数据
        DateTime now = DateUtil.date();
        List<DateTime> rangeToList = DateUtil.rangeToList(beginTime, endTime, yearFlag ? DateField.MONTH : DateField.DAY_OF_YEAR);
        List<GetElectricityStatisticsOutput> outputs = new ArrayList<>(rangeToList.size());
        GetElectricityStatisticsOutput output;
        for (DateTime dateTime : rangeToList) {
            if (DateUtil.compare(dateTime, now) > 0) {
                break;
            }
            output = new GetElectricityStatisticsOutput();
            if (isUse) {
                output.setCurrentPeriodUse(currentMap.getOrDefault(dateTime, BigDecimal.ZERO));
                output.setLastPeriodUse(lastMap.getOrDefault(dateTime, BigDecimal.ZERO));
            } else {
                output.setCurrentPeriodGenerate(currentMap.getOrDefault(dateTime, BigDecimal.ZERO));
                output.setLastPeriodGenerate(lastMap.getOrDefault(dateTime, BigDecimal.ZERO));
            }
            output.setDateStr(yearFlag ? dateTime.toString(BusinessConstant.DATE_FORMAT_MONTH) : dateTime.toDateStr());
            outputs.add(output);
        }
        return outputs;
    }

    /**
     * 获取线路名称集合
     *
     * @param context 上下文
     * @param ouId    你id
     * @return {@link Set }<{@link String }>
     */
    private Set<String> getLineNameSet(XContext context, String ouId) {
        List<DynamicQueryBasePowerLineOutput> lineList = ServiceUtil.getBasePowerLineListByOuIdList(context, ServiceUtil.getOrgIdsByPath(context, ouId));
        Set<String> lineNameSet;
        if (CollUtil.isEmpty(lineList)) {
            lineNameSet = new HashSet<>(2);
            lineNameSet.add(BusinessConstant.DEFAULT_VALUE);
        } else {
            lineNameSet = lineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName)
                    .collect(Collectors.toSet());
        }
        return lineNameSet;
    }

    /**
     * 获取电站日统计数据
     *
     * @param context     上下文
     * @param beginTime   开始时间
     * @param endTime     结束时间
     * @param lineNameSet 行名称集
     * @return {@link StationDailyProductionSituationEnt }
     */
    private StationDailyProductionSituationEnt getStationDailyStatistics(XContext context, Date beginTime, Date endTime, Set<String> lineNameSet) {
        QueryWrapper<StationDailyProductionSituationEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("NVL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                    "NVL( SUM( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION");
        } else {
            queryWrapper.select("IFNULL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                    "IFNULL( SUM( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION");
        }
        StationDailyProductionSituationMapper mapper = context.getBean(StationDailyProductionSituationMapper.class);
        return mapper.selectOne(queryWrapper.lambda()
                .between(StationDailyProductionSituationEnt::getDataDate, beginTime, endTime)
                .in(StationDailyProductionSituationEnt::getStationName, lineNameSet)
        );
    }

    /**
     * 获取电站日统计数据列表
     *
     * @param context     上下文
     * @param beginTime   开始时间
     * @param endTime     结束时间
     * @param lineNameSet 行名称集
     * @param yearFlag    是否按年展示
     * @return {@link List }<{@link StationDailyProductionSituationEnt }>
     */
    private List<StationDailyProductionSituationEnt> getStationDailyStatisticsList(XContext context, Date beginTime, Date endTime, Set<String> lineNameSet, boolean yearFlag) {
        QueryWrapper<StationDailyProductionSituationEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("NVL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                            "NVL( SUM( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                            yearFlag ? "TO_CHAR( DATA_DATE, 'YYYY-MM-' ) || '01 00:00:00' AS DATA_DATE" : "DATA_DATE")
                    .last(yearFlag ? "GROUP BY TO_CHAR( DATA_DATE, 'YYYY-MM-' )" : "GROUP BY DATA_DATE");
        } else {
            queryWrapper.select("IFNULL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                            "IFNULL( SUM( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                            yearFlag ? "DATE_FORMAT( DATA_DATE, '%Y-%M-01' ) AS DATA_DATE" : "DATA_DATE")
                    .last(yearFlag ? "GROUP BY DATE_FORMAT( DATA_DATE, '%Y-%m-01')" : "GROUP BY DATA_DATE");
        }
        return context.getBean(StationDailyProductionSituationMapper.class).selectList(queryWrapper.lambda()
                .between(StationDailyProductionSituationEnt::getDataDate, beginTime, endTime)
                .in(StationDailyProductionSituationEnt::getStationName, lineNameSet)
        );
    }

    /*------------------------------ 能耗分析(长庆) ------------------------------*/

    /**
     * 今日/昨日/同期电量统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link PowerStatisticsOutput }>
     */
    @XText("能耗分析--今日/昨日/同期电量统计")
    @XApiGet
    public XListResult<PowerStatisticsOutput> powerStatistics(XContext context, GetEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        }
        List<PowerStatisticsOutput> outputs;
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, plantInput);
        //电站名称列表有值再查
        if (CollUtil.isNotEmpty(plantList)) {
            DateTime today = DateUtil.beginOfDay(DateUtil.date());
            String oilFieldCode = ServiceUtil.getOilFieldCode(context);
            List<String> collect = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                    .collect(Collectors.toList());
            ThirdDailyAccumulationUpdateViewMapper mapper = context.getBean(ThirdDailyAccumulationUpdateViewMapper.class);
            List<ThirdDailyAccumulationUpdateView> viewList = mapper.selectPowerStatisticsByDate(ThirdDailyAccumulationUpdateView.builder()
                    .today(today)
                    .yesterday(DateUtil.beginOfDay(DateUtil.yesterday()))
                    .lastYear(DateUtil.offset(today, DateField.YEAR, -1))
                    .stationNameList(collect)
                    .systemSource(oilFieldCode)
                    .build());
            outputs = XCopyUtils.copyNewList(viewList, PowerStatisticsOutput.class);
            //谷电占比
            if (BusinessConstant.ENV_SY.equals(oilFieldCode)) {
                IDailyElectricityTrendCloudService service = context.getBean(IDailyElectricityTrendCloudService.class);
                XListResult<GetValleyElectricRateOutput> result = service.queryValleyElectricRate(context, GetLineDailyElectricityTrendInput.builder()
                        .systemSource(oilFieldCode)
                        .stationNameList(collect)
                        .startTime(today)
                        .build());
                result.throwIfFail();
                List<GetValleyElectricRateOutput> valleyElectricRateList = result.getResult();
                Map<Integer, BigDecimal> decimalMap = valleyElectricRateList.stream()
                        .collect(Collectors.toMap(GetValleyElectricRateOutput::getDayFlag, GetValleyElectricRateOutput::getValleyElectricRate));
                for (PowerStatisticsOutput output : outputs) {
                    output.setValleyElectricRate(decimalMap.getOrDefault(output.getDayFlag(), BigDecimal.ZERO));
                }
            }
        } else {
            outputs = new ArrayList<>(4);
            for (int i = 1; i < 4; i++) {
                outputs.add(
                        PowerStatisticsOutput.builder()
                                .dayFlag(i)
                                .photovoltaicPower(BigDecimal.ZERO)
                                .dailyElectricityConsumption(BigDecimal.ZERO)
                                .avoidPeakRate(BigDecimal.ZERO)
                                .greenElectricityRate(BigDecimal.ZERO)
                                .valleyElectricRate(BigDecimal.ZERO)
                                .build()
                );
            }
        }
        return XListResult.success(outputs);
    }

    /**
     * 用能分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerAnalyseOutput }>
     */
    @XText("能耗分析--用能分析")
    @XApiGet
    public XListResult<GetPowerAnalyseOutput> powerAnalyse(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLineWellheadInput wellheadInput = new DynamicQueryBasePowerLineWellheadInput();
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            wellheadInput.setLineName(stationName);
            plantInput.setLineName(stationName);
        } else {
            List<String> orgIds = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            wellheadInput.setOuIds(orgIds);
            plantInput.setOuIds(orgIds);
        }
        input.getBetweenDate();
        DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //用电功率
        Map<Date, BigDecimal> avgMap = this.queryPowerConsumptionHour(context,
                ServiceUtil.getPowerLineWellheadList(context, wellheadInput),
                input.getStartTime(),
                endTime,
                oilFieldCode,
                1
        );
        //发电功率
        Map<Date, BigDecimal> powerMap = this.queryGeneratedPowerHour(context,
                ServiceUtil.getPowerLinePlantList(context, plantInput),
                input.getStartTime(),
                endTime,
                oilFieldCode,
                1
        );
        //封装数据
        return XListResult.success(this.setPowerAnalyseList(avgMap, powerMap, input.getStartTime(), input.getEndTime(), oilFieldCode, 1));
    }

    /**
     * 发电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerGenerationTrendOutput }>
     */
    @XText("能耗分析--发电趋势")
    @XApiGet
    public XListResult<GetPowerGenerationTrendOutput> powerGenerationTrend(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        }
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, plantInput);
        Map<Date, BigDecimal> nowMap;
        Map<Date, BigDecimal> lastMap;
        input.getBetweenDate();
        //电站名称列表有值再查
        if (CollUtil.isNotEmpty(plantList)) {
            List<String> collect = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                    .collect(Collectors.toList());
            String oilFieldCode = ServiceUtil.getOilFieldCode(context);
            //查本期
            DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
            nowMap = this.getPowerMap(context, input.getStartTime(), endTime, collect, 1, oilFieldCode, null);
            lastMap = this.getPowerMap(context, DateUtil.offset(input.getStartTime(), DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), collect, 1, oilFieldCode, null);
        } else {
            nowMap = Collections.emptyMap();
            lastMap = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetPowerGenerationTrendOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        BigDecimal powerGeneration;
        BigDecimal lastPowerGeneration;
        for (DateTime dateTime : rangeToList) {
            if (nowMap.containsKey(dateTime)) {
                powerGeneration = nowMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerGeneration = BigDecimal.ZERO;
            }
            if (lastMap.containsKey(DateUtil.offset(dateTime, DateField.YEAR, -1))) {
                lastPowerGeneration = lastMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                lastPowerGeneration = BigDecimal.ZERO;
            }
            outputs.add(GetPowerGenerationTrendOutput.builder()
                    .dateFormat(dateTime.toString())
                    .powerGeneration(powerGeneration)
                    .lastPowerGeneration(lastPowerGeneration)
                    .ratio(BaseUtils.getRate(powerGeneration, lastPowerGeneration))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 用电趋势
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetElectricityConsumptionTrendOutput }>
     */
    @XText("能耗分析--用电趋势")
    @XApiGet
    public XListResult<GetElectricityConsumptionTrendOutput> electricityConsumptionTrend(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        }
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, plantInput);
        Map<Date, BigDecimal> nowMap;
        Map<Date, BigDecimal> lastMap;
        input.getBetweenDate();
        //电站名称列表有值再查
        if (CollUtil.isNotEmpty(plantList)) {
            List<String> collect = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                    .collect(Collectors.toList());
            //查本期
            String oilFieldCode = ServiceUtil.getOilFieldCode(context);
            DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
            nowMap = this.getPowerMap(context, input.getStartTime(), endTime, collect, 2, oilFieldCode, null);
            lastMap = this.getPowerMap(context, DateUtil.offset(input.getStartTime(), DateField.YEAR, -1), DateUtil.offset(endTime, DateField.YEAR, -1), collect, 2, oilFieldCode, null);
        } else {
            nowMap = Collections.emptyMap();
            lastMap = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetElectricityConsumptionTrendOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        BigDecimal photovoltaicConsumption;
        BigDecimal lastPhotovoltaicConsumption;
        for (DateTime dateTime : rangeToList) {
            if (nowMap.containsKey(dateTime)) {
                photovoltaicConsumption = nowMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                photovoltaicConsumption = BigDecimal.ZERO;
            }
            if (lastMap.containsKey(DateUtil.offset(dateTime, DateField.YEAR, -1))) {
                lastPhotovoltaicConsumption = lastMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                lastPhotovoltaicConsumption = BigDecimal.ZERO;
            }
            outputs.add(GetElectricityConsumptionTrendOutput.builder()
                    .dateFormat(dateTime.toString())
                    .photovoltaicConsumption(photovoltaicConsumption)
                    .lastPhotovoltaicConsumption(lastPhotovoltaicConsumption)
                    .ratio(BaseUtils.getRate(photovoltaicConsumption, lastPhotovoltaicConsumption))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 光伏/市电用电占比趋势分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetElectricityAnalysisOutput }>
     */
    @XText("能耗分析--光伏/市电用电占比趋势分析")
    @XApiGet
    public XListResult<GetElectricityAnalysisOutput> electricityAnalysis(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        }
        input.getBetweenDate();
        Map<Date, GetLineDailyElectricityTrendOutput> collect;
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, plantInput);
        if (CollUtil.isNotEmpty(plantList)) {
            DateTime endTime = DateUtil.offsetDay(input.getEndTime(), 1);
            collect = this.getDailyElectricityTrendMap(context, GetLineDailyElectricityTrendInput.builder()
                    .startTime(input.getStartTime())
                    .endTime(endTime)
                    .stationNameList(plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()))
                    .systemSource(ServiceUtil.getOilFieldCode(context))
                    .build());
        } else {
            collect = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetElectricityAnalysisOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        GetLineDailyElectricityTrendOutput getLineDailyElectricityTrendOutput;
        BigDecimal photovoltaicConsumption = BigDecimal.ZERO;
        BigDecimal municipalConsumption = BigDecimal.ZERO;
        BigDecimal lastPhotovoltaicConsumption = BigDecimal.ZERO;
        BigDecimal lastMunicipalConsumption = BigDecimal.ZERO;
        for (DateTime dateTime : rangeToList) {
            if (collect.containsKey(dateTime)) {
                getLineDailyElectricityTrendOutput = collect.get(dateTime);
                photovoltaicConsumption = getLineDailyElectricityTrendOutput.getPhotovoltaicPower();
                municipalConsumption = getLineDailyElectricityTrendOutput.getMunicipalPower();
                lastPhotovoltaicConsumption = getLineDailyElectricityTrendOutput.getLastPhotovoltaicPower();
                lastMunicipalConsumption = getLineDailyElectricityTrendOutput.getLastMunicipalPower();
            }
            outputs.add(GetElectricityAnalysisOutput.builder()
                    .dateFormat(dateTime.toString())
                    .photovoltaicConsumption(photovoltaicConsumption)
                    .municipalConsumption(municipalConsumption)
                    .lastPhotovoltaicConsumption(lastPhotovoltaicConsumption)
                    .lastMunicipalConsumption(lastMunicipalConsumption)
                    .ratio(BaseUtils.getRate(photovoltaicConsumption, lastPhotovoltaicConsumption))
                    .build());
        }
        return XListResult.success(outputs);
    }

    /*------------------------------ 井场实时分析 ------------------------------*/

    /**
     * 发电详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetPowerGenerationDetailOutput }>
     */
    @XText("井场实时分析--发电详情")
    @XApiGet
    public XSingleResult<GetPowerGenerationDetailOutput> powerGenerationDetail(XContext context, GetEnergyConsumptionAnalysisInput input) {
        //光伏装机总量(kWh)
        BigDecimal photovoltaicPower = BigDecimal.ZERO;
        //发电功率(kWh)
        BigDecimal powerGeneration;
        //累计发电量(kWh)
        BigDecimal accumulatePowerGeneration;
        //就地消纳电量(KWh)
        BigDecimal inPlaceConsumption;
        //井场额定功率
        BigDecimal operatingLoad = BigDecimal.ZERO;
        //开井数量
        int openWellNumber;
        //日累计产液量(m³)
        BigDecimal dailyLiquidProduction;
        //日累计用电量(KW·h)
        BigDecimal dailyElectricityConsumption;
        //上网电量
        BigDecimal avoidPeakRate;
        //用电功率(kWh)
        BigDecimal powerConsumption;
        //绿电占比
        BigDecimal greenElectricityRate;
        List<String> orgIds = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .ouIds(orgIds)
                .build());
        Set<String> plantSet = new HashSet<>(plantList.size());
        for (DynamicQueryBasePowerLinePlantViewOutput plant : plantList) {
            photovoltaicPower = photovoltaicPower.add(plant.getTotalPower());
            plantSet.add(plant.getStationName());
        }
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        //累计发电
        QueryWrapper<ThirdDailyAccumulationUpdateEnt> queryWrapper = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper.select("NVL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                    "NVL( SUM( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                    "NVL( SUM( DAILY_LIQUID_PRODUCTION ), 0 ) AS DAILY_LIQUID_PRODUCTION",
                    "NVL( SUM( IN_PLACE_CONSUMPTION ), 0 ) AS IN_PLACE_CONSUMPTION");
        } else {
            queryWrapper.select("IFNULL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER",
                    "IFNULL( SUM( DAILY_ELECTRICITY_CONSUMPTION ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                    "IFNULL( SUM( DAILY_LIQUID_PRODUCTION ), 0 ) AS DAILY_LIQUID_PRODUCTION",
                    "IFNULL( SUM( IN_PLACE_CONSUMPTION ), 0 ) AS IN_PLACE_CONSUMPTION");
        }
        ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
        ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(queryWrapper.lambda()
                .eq(ThirdDailyAccumulationUpdateEnt::getSaveDate, today)
                .in(ThirdDailyAccumulationUpdateEnt::getStationName, plantSet)
        );
        accumulatePowerGeneration = analysisEnt.getPhotovoltaicPower();
        inPlaceConsumption = analysisEnt.getInPlaceConsumption();
        dailyLiquidProduction = analysisEnt.getDailyLiquidProduction();
        dailyElectricityConsumption = analysisEnt.getDailyElectricityConsumption();
        greenElectricityRate = BaseUtils.getRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption());
        avoidPeakRate = BaseUtils.divide(analysisEnt.getPhotovoltaicPower(), analysisEnt.getInPlaceConsumption());
        //发电功率
        QueryWrapper<ThirdActivePowerDailyUpdateEnt> queryWrapper1 = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper1.select("NVL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER");
        } else {
            queryWrapper1.select("IFNULL( SUM( PHOTOVOLTAIC_POWER ), 0 ) AS PHOTOVOLTAIC_POWER");
        }
        ThirdActivePowerDailyUpdateMapper activeMapper = context.getBean(ThirdActivePowerDailyUpdateMapper.class);
        ThirdActivePowerDailyUpdateEnt activeEnt = activeMapper.selectOne(queryWrapper1.lambda()
                .eq(ThirdActivePowerDailyUpdateEnt::getSaveDate, today)
                .in(ThirdActivePowerDailyUpdateEnt::getStationName, plantSet)
        );
        powerGeneration = activeEnt.getPhotovoltaicPower();
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = ServiceUtil.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                .ouIds(orgIds)
                .build());
        Set<String> wellSet = new HashSet<>(wellheadList.size());
        for (DynamicQueryBasePowerLineWellheadViewOutput well : wellheadList) {
            operatingLoad = operatingLoad.add(well.getRatedPower());
            wellSet.add(well.getWellNumber());
        }
        //用电功率
        QueryWrapper<ThirdWellAvgActivePowerEnt> queryWrapper2 = new QueryWrapper<>();
        if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
            queryWrapper2.select("NVL( SUM( AVG_ACTIVE_POWER ), 0 ) AS AVG_ACTIVE_POWER")
                    .apply("TO_CHAR( INPUT_TIME, 'YYYY-MM-DD' ) = {0}", today.toDateStr());
        } else {
            queryWrapper2.select("IFNULL( SUM( AVG_ACTIVE_POWER ), 0 ) AS AVG_ACTIVE_POWER")
                    .apply("DATE( INPUT_TIME ) = {0}", today);
        }
        ThirdWellAvgActivePowerMapper avgMapper = context.getBean(ThirdWellAvgActivePowerMapper.class);
        ThirdWellAvgActivePowerEnt activePowerEnt = avgMapper.selectOne(queryWrapper2.lambda()
                .in(ThirdWellAvgActivePowerEnt::getWellNumber, wellSet)
        );
        powerConsumption = activePowerEnt.getAvgActivePower();
        //开井数量
        ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
        List<ThirdCurrentWellConditionEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                .in(ThirdCurrentWellConditionEnt::getWellNumber, wellSet)
        );
        openWellNumber = (int) list.stream()
                .filter(l -> CharSequenceUtil.equals(l.getWellStatus(), BusinessConstant.OPEN_WELL))
                .count();
        return XSingleResult.success(GetPowerGenerationDetailOutput.builder()
                .photovoltaicPower(photovoltaicPower)
                .powerGeneration(powerGeneration)
                .accumulatePowerGeneration(accumulatePowerGeneration)
                .inPlaceConsumption(inPlaceConsumption)
                .operatingLoad(operatingLoad)
                .openWellNumber(openWellNumber)
                .dailyLiquidProduction(dailyLiquidProduction)
                .dailyElectricityConsumption(dailyElectricityConsumption)
                .avoidPeakRate(avoidPeakRate)
                .powerConsumption(powerConsumption)
                .greenElectricityRate(greenElectricityRate)
                .build());
    }

    /**
     * 井场实时监控
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWellFieldSituationOutput }>
     */
    @XText("井场实时分析--井场实时监控")
    @XApiGet
    public XSingleResult<GetWellFieldSituationOutput> wellFieldSituation(XContext context, GetEnergyConsumptionAnalysisInput input) {
        int sumWellNumber = 0;
        int normallyOpenWellNumber = 0;
        int interOpenWellNumber = 0;
        int openWellNumber = 0;
        int stopWellNumber = 0;
        List<GetThirdCurrentWellConditionOutput> currentList = Collections.emptyList();
        GetWellFieldSituationOutput output = new GetWellFieldSituationOutput();
        List<DynamicQueryBaseWellheadOutput> wellList = this.getWellListByOuId(context, ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        if (CollUtil.isNotEmpty(wellList)) {
            sumWellNumber = wellList.size();
            Set<String> set = new HashSet<>(wellList.size());
            for (DynamicQueryBaseWellheadOutput out : wellList) {
                set.add(out.getWellNumber());
                if (CharSequenceUtil.equals(out.getRunTypeKey(), BusinessConstant.INTERVAL_PUMPING_WELL)) {
                    interOpenWellNumber++;
                } else {
                    normallyOpenWellNumber++;
                }
            }
            ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
            List<ThirdCurrentWellConditionEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                    .in(ThirdCurrentWellConditionEnt::getWellNumber, set)
            );
            if (CollUtil.isNotEmpty(list)) {
                //查生效中的所有井口
                ISpaceInstitutionDetailCloudService service = context.getBean(ISpaceInstitutionDetailCloudService.class);
                XListResult<GetSpaceInstitutionWellheadOutput> result = service.getInstitutionWellheadList(context, new GetSpaceInstitutionWellheadInput());
                result.throwIfFail();
                List<GetSpaceInstitutionWellheadOutput> wellheadOutputList = result.getResult();
                Map<String, BigDecimal> collect;
                if (CollUtil.isNotEmpty(wellheadOutputList)) {
                    collect = wellheadOutputList.stream()
                            .collect(Collectors.toMap(GetSpaceInstitutionWellheadOutput::getWellNumber,
                                    GetSpaceInstitutionWellheadOutput::getRunDuration));
                } else {
                    collect = Collections.emptyMap();
                }
                BigDecimal surplusTime;
                currentList = new ArrayList<>(list.size());
                for (ThirdCurrentWellConditionEnt ent : list) {
                    if (CharSequenceUtil.equals(ent.getWellStatus(), BusinessConstant.OPEN_WELL)) {
                        openWellNumber++;
                    } else {
                        stopWellNumber++;
                    }
                    Date nextTime = ent.getNextTime();
                    if (ObjectUtil.isNotNull(nextTime) && CharSequenceUtil.contains(DateUtil.date(ent.getNextTime()).toString(), ThirdPartyApiConstant.DATE_FLAG)) {
                        nextTime = null;
                    }
                    if (collect.containsKey(ent.getWellNumber())) {
                        surplusTime = collect.get(ent.getWellNumber()).subtract(ent.getRunningTime());
                    } else {
                        surplusTime = BigDecimal.ZERO;
                    }
                    currentList.add(
                            GetThirdCurrentWellConditionOutput.builder()
                                    .wellNumber(ent.getWellNumber())
                                    .nextTime(nextTime)
                                    .wellStatus(ent.getWellStatus())
                                    .cumulativeProduction(ent.getCumulativeProduction())
                                    .runningTime(ent.getRunningTime())
                                    .surplusTime(surplusTime)
                                    .build()
                    );
                }
            }
        }
        output.setList(currentList);
        output.setSumWellNumber(sumWellNumber);
        output.setNormallyOpenWellNumber(normallyOpenWellNumber);
        output.setInterOpenWellNumber(interOpenWellNumber);
        output.setOpenWellNumber(openWellNumber);
        output.setStopWellNumber(stopWellNumber);
        return XSingleResult.success(output);
    }

    /**
     * 本日用电对比
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerAnalyseOutput }>
     */
    @XText("井场实时分析--本日用电对比")
    @XApiGet
    public XListResult<GetPowerAnalyseOutput> electricityConsumptionContrast(XContext context, GetEnergyConsumptionAnalysisInput input) {
        List<String> orgIds = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        DateTime tomorrow = DateUtil.beginOfDay(DateUtil.tomorrow());
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        //用电功率
        Map<Date, BigDecimal> avgMap = this.queryPowerConsumptionHour(context,
                ServiceUtil.getPowerLineWellheadList(context, DynamicQueryBasePowerLineWellheadInput.builder()
                        .ouIds(orgIds)
                        .build()),
                today,
                tomorrow,
                oilFieldCode,
                1
        );
        //发电功率
        Map<Date, BigDecimal> powerMap = this.queryGeneratedPowerHour(context,
                ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                        .ouIds(orgIds)
                        .build()),
                today,
                tomorrow,
                oilFieldCode,
                1
        );
        //封装数据
        return XListResult.success(this.setPowerAnalyseList(avgMap, powerMap, today, tomorrow, oilFieldCode, 1));
    }

    /**
     * 本日预测/实际功率对比
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPredictedPowerContrastOutput }>
     */
    @XText("井场实时分析--本日预测/实际功率对比")
    @XApiGet
    public XListResult<GetPredictedPowerContrastOutput> predictedPowerContrast(XContext context, GetEnergyConsumptionAnalysisInput input) {
        Map<String, BigDecimal> shortMap;
        Map<Integer, BigDecimal> powerMap;
        DateTime today = DateUtil.beginOfDay(DateUtil.date());
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, DynamicQueryBasePowerLinePlantInput.builder()
                .ouIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()))
                .build());
        if (CollUtil.isNotEmpty(plantList)) {
            List<String> plantIds = plantList.stream()
                    .map(DynamicQueryBasePowerLinePlantViewOutput::getPlantId)
                    .collect(Collectors.toList());
            //预测发电量
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            shortMap = mapper.selectShortPowerHourList(PredictedPowerView.builder()
                            .createTime(today)
                            .plantIds(plantIds)
                            .build()).stream()
                    .collect(Collectors.toMap(PredictedPowerView::getHourTime, PredictedPowerView::getPower));
            //实际功率
            ThirdActivePowerDailyUpdateViewMapper nowMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
            powerMap = nowMapper.selectEnergyUseCurve(ThirdActivePowerDailyUpdateView.builder()
                            .stationIds(plantIds)
                            .saveDate(today)
                            .createDate(DateUtil.date())
                            .build()).stream()
                    .collect(Collectors.toMap(ThirdActivePowerDailyUpdateView::getHourNumber, ThirdActivePowerDailyUpdateView::getPhotovoltaicPower));
        } else {
            shortMap = Collections.emptyMap();
            powerMap = Collections.emptyMap();
        }
        //封装数据
        List<GetPredictedPowerContrastOutput> outputs = new ArrayList<>(24);
        BigDecimal predictPower;
        BigDecimal actualPower;
        for (int i = 0; i < 24; i++) {
            String value = String.valueOf(i);
            predictPower = shortMap.getOrDefault(value, BigDecimal.ZERO);
            actualPower = powerMap.getOrDefault(i, BigDecimal.ZERO);
            outputs.add(
                    GetPredictedPowerContrastOutput.builder()
                            .hourNumber(i)
                            .predictPower(predictPower)
                            .actualPower(actualPower)
                            .build()
            );
        }
        return XListResult.success(outputs);
    }

    /*------------------------------ 间开效果评价 ------------------------------*/

    /**
     * 月度总览
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetMonthlyOverviewOutput }>
     */
    @XText("间开效果评价--月度总览")
    @XApiGet
    public XSingleResult<GetMonthlyOverviewOutput> monthlyOverview(XContext context, GetEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        }
        BigDecimal accumulatePowerGeneration = BigDecimal.ZERO;
        BigDecimal dailyElectricityConsumption = BigDecimal.ZERO;
        BigDecimal greenElectricityRate = BigDecimal.ZERO;
        BigDecimal economicBenefit = BigDecimal.ZERO;
        BigDecimal carbonReduction = BigDecimal.ZERO;
        DateTime end = DateUtil.beginOfDay(DateUtil.date());
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, plantInput);
        if (CollUtil.isNotEmpty(plantList)) {
            QueryWrapper<ThirdDailyAccumulationUpdateEnt> queryWrapper = new QueryWrapper<>();
            if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
                queryWrapper.select("NVL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "NVL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                        "NVL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                        "NVL( SUM( in_place_consumption ), 0 ) AS in_place_consumption");
            } else {
                queryWrapper.select("IFNULL( SUM( photovoltaic_power ), 0 ) AS photovoltaic_power",
                        "IFNULL( SUM( daily_electricity_consumption ), 0 ) AS daily_electricity_consumption",
                        "IFNULL( SUM( daily_liquid_production ), 0 ) AS daily_liquid_production",
                        "IFNULL( SUM( in_place_consumption ), 0 ) AS in_place_consumption");
            }
            DateTime start = DateUtil.beginOfMonth(end);
            //累计发电
            ThirdDailyAccumulationUpdateMapper analysisMapper = context.getBean(ThirdDailyAccumulationUpdateMapper.class);
            ThirdDailyAccumulationUpdateEnt analysisEnt = analysisMapper.selectOne(queryWrapper.lambda()
                    .between(ThirdDailyAccumulationUpdateEnt::getSaveDate, start, end)
                    .in(ThirdDailyAccumulationUpdateEnt::getStationName, plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()))
            );
            accumulatePowerGeneration = analysisEnt.getPhotovoltaicPower();
            dailyElectricityConsumption = analysisEnt.getDailyElectricityConsumption();
            greenElectricityRate = BaseUtils.getRate(analysisEnt.getInPlaceConsumption(), analysisEnt.getDailyElectricityConsumption());
            economicBenefit = ServiceUtil.getEconomicBenefit(analysisEnt.getPhotovoltaicPower(), BusinessConstant.ENV_CQ);
            carbonReduction = ServiceUtil.calculateCarbonReduction(analysisEnt.getPhotovoltaicPower());
        }
        ISpaceOptimizeShortCloudService service = context.getBean(ISpaceOptimizeShortCloudService.class);
        XSingleResult<GetMonthlyWellCountOutput> result = service.monthlyWellCount(context, GetMonthlyWellCountInput.builder()
                .month(end.monthBaseOne())
                .build());
        result.throwIfFail();
        Long wellNumber = result.getResult().getWellNumber();
        return XSingleResult.success(GetMonthlyOverviewOutput.builder()
                .accumulatePowerGeneration(accumulatePowerGeneration)
                .dailyElectricityConsumption(dailyElectricityConsumption)
                .greenElectricityRate(greenElectricityRate)
                .wellNumber(wellNumber)
                .economicBenefit(economicBenefit)
                .carbonReduction(carbonReduction)
                .build());
    }

    /**
     * 本月累计节电经济效益/减碳量
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetEconomicBenefitsOutput }>
     */
    @XText("间开效果评价--本月累计节电经济效益/减碳量")
    @XApiGet
    public XListResult<GetEconomicBenefitsOutput> economicBenefitCarbonReduction(XContext context, GetEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            plantInput.setLineName(stationName);
        } else {
            plantInput.setOuIds(ServiceUtil.getOrgIdsByPath(context, input.getOuId()));
        }
        List<DynamicQueryBasePowerLinePlantViewOutput> plantList = ServiceUtil.getPowerLinePlantList(context, plantInput);
        DateTime end = DateUtil.beginOfDay(DateUtil.yesterday());
        DateTime start;
        Map<Date, BigDecimal> map;
        //区分环境
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        List<DateTime> rangeToList;
        Integer dateType = ObjectUtil.defaultIfNull(input.getDateType(), 0);
        if (dateType.equals(1)) {
            start = DateUtil.beginOfYear(end);
            rangeToList = DateUtil.rangeToList(start, end, DateField.MONTH);
        } else {
            start = DateUtil.beginOfMonth(end);
            rangeToList = DateUtil.rangeToList(start, end, DateField.DAY_OF_YEAR);
        }
        if (CollUtil.isNotEmpty(plantList)) {
            //实际发电量
            map = this.getPowerMap(context, start, DateUtil.offsetDay(end, 1),
                    plantList.stream()
                            .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                            .collect(Collectors.toList()), 1, oilFieldCode, dateType);
        } else {
            map = Collections.emptyMap();
        }
        //封装数据
        List<GetEconomicBenefitsOutput> outputs = new ArrayList<>(rangeToList.size());
        BigDecimal economicBenefit;
        BigDecimal carbonReduction;
        BigDecimal electricityFees;
        BigDecimal saveElectricity;
        switch (oilFieldCode) {
            case BusinessConstant.ENV_CQ:
                for (DateTime dateTime : rangeToList) {
                    if (map.containsKey(dateTime)) {
                        economicBenefit = ServiceUtil.getEconomicBenefit(map.get(dateTime), oilFieldCode);
                        carbonReduction = ServiceUtil.calculateCarbonReduction(map.get(dateTime));
                    } else {
                        economicBenefit = BigDecimal.ZERO;
                        carbonReduction = BigDecimal.ZERO;
                    }
                    outputs.add(GetEconomicBenefitsOutput.builder()
                            .dateFormat(dateTime)
                            .economicBenefit(economicBenefit)
                            .carbonReduction(carbonReduction)
                            .build());
                }
                break;
            case BusinessConstant.ENV_SY:
                for (DateTime dateTime : rangeToList) {
                    if (map.containsKey(dateTime)) {
                        saveElectricity = map.get(dateTime);
                        carbonReduction = ServiceUtil.calculateCarbonReduction(map.get(dateTime));
                        electricityFees = ServiceUtil.getEconomicBenefit(map.get(dateTime), oilFieldCode);
                    } else {
                        saveElectricity = BigDecimal.ZERO;
                        carbonReduction = BigDecimal.ZERO;
                        electricityFees = BigDecimal.ZERO;
                    }
                    outputs.add(GetEconomicBenefitsOutput.builder()
                            .dateFormat(dateTime)
                            .carbonReduction(carbonReduction)
                            .saveElectricity(saveElectricity)
                            .electricityFees(electricityFees)
                            .build());
                }
                break;
            default:
        }
        return XListResult.success(outputs);
    }

    /**
     * 井口分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult }<{@link GetWellheadAnalysisOutput }>
     */
    @XText("间开效果评价--井口分析")
    @XApiGet
    public XPageResult<GetWellheadAnalysisOutput> wellheadAnalysis(XContext context, GetWellheadAnalysisInput input) {
        DynamicQueryBasePowerLineWellheadInput wellheadInput = new DynamicQueryBasePowerLineWellheadInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            wellheadInput.setLineName(stationName);
        } else {
            List<String> orgIds = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            wellheadInput.setOuIds(orgIds);
        }
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = ServiceUtil.getPowerLineWellheadList(context, wellheadInput);
        if (CollUtil.isNotEmpty(wellheadList)) {
            DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
            IDailyElectricityTrendCloudService service = context.getBean(IDailyElectricityTrendCloudService.class);
            GetWellDailyElectricityTrendInput build = GetWellDailyElectricityTrendInput.builder()
                    .createDate(yesterday)
                    .wellNumberList(wellheadList.stream().map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                            .collect(Collectors.toList()))
                    .wellNumber(input.getWellNumber())
                    .build();
            build.setPage(input.getPage());
            build.setLimit(input.getLimit());
            XPageResult<GetWellDailyElectricityTrendOutput> result = service.queryWellheadAnalysisPage(context, build);
            result.throwIfFail();
            XPageData<GetWellDailyElectricityTrendOutput> pageInfo = result.getResult();
            List<GetWellDailyElectricityTrendOutput> list = pageInfo.getItems();
            if (CollUtil.isNotEmpty(list)) {
                return XPageResult.success(XCopyUtils.copyNewList(list, GetWellheadAnalysisOutput.class), input, pageInfo.getTotal());
            }
        }
        return XPageResult.success(Collections.emptyList(), input, 0);
    }

    /*------------------------------ 井组监控 ------------------------------*/

    /**
     * 光伏电站发电详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPowerGenerationDetailsOutput }>
     */
    @XText("井组监控--光伏电站发电详情")
    @XApiPost
    public XListResult<GetPowerGenerationDetailsOutput> powerGenerationDetails(XContext context, GetEnergyConsumptionAnalysisInput input) {
        Map<Date, BigDecimal> nowMap;
        Map<Date, BigDecimal> lastMap;
        DateTime begin = DateUtil.beginOfMonth(DateUtil.date());
        DateTime end = DateUtil.endOfMonth(begin);
        //查电站关联井口
        DynamicQueryBasePowerLineInput lineParam = new DynamicQueryBasePowerLineInput();
        if (CharSequenceUtil.isNotBlank(input.getStationName())) {
            //用stationName匹配电站
            lineParam.setLineNames(CollUtil.list(false, input.getStationName()));
        } else {
            List<String> orgIdsByPath = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            lineParam.setOuIdList(orgIdsByPath);
        }
        //查电站列表,计算统计数据
        List<DynamicQueryBasePowerLineOutput> powerLineList = ServiceUtil.getPowerLineList(context, lineParam);
        if (CollUtil.isNotEmpty(powerLineList)) {
            String oilFieldCode = ServiceUtil.getOilFieldCode(context);
            List<String> lineNameSet = powerLineList.stream()
                    .map(DynamicQueryBasePowerLineOutput::getLineName).collect(Collectors.toList());
            //查本期实际发电量
            nowMap = this.getPowerMap(context, begin, end, lineNameSet, 1, oilFieldCode, null);
            //查同期实际发电量
            DateTime offset = DateUtil.offset(begin, DateField.YEAR, -1);
            lastMap = this.getPowerMap(context, offset, DateUtil.endOfMonth(offset), lineNameSet, 1, oilFieldCode, null);
        } else {
            nowMap = Collections.emptyMap();
            lastMap = Collections.emptyMap();
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(begin, end, DateField.DAY_OF_YEAR);
        List<GetPowerGenerationDetailsOutput> outputs = new ArrayList<>(rangeToList.size());
        //封装数据
        BigDecimal powerGeneration;
        BigDecimal lastPowerGeneration;
        for (DateTime dateTime : rangeToList) {
            if (nowMap.containsKey(dateTime)) {
                powerGeneration = nowMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerGeneration = BigDecimal.ZERO;
            }
            if (lastMap.containsKey(DateUtil.offset(dateTime, DateField.YEAR, -1))) {
                lastPowerGeneration = lastMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                lastPowerGeneration = BigDecimal.ZERO;
            }
            outputs.add(GetPowerGenerationDetailsOutput.builder()
                    .dateFormat(dateTime.toString())
                    .powerGeneration(powerGeneration)
                    .lastPowerGeneration(lastPowerGeneration)
                    .build());
        }
        return XListResult.success(outputs);
    }

    /**
     * 井组生产情况
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult }<{@link GetWellProductionStatusOutput }>
     */
    @XText("井组监控--井组生产情况")
    @XApiPost
    public XSingleResult<GetWellProductionStatusOutput> wellProductionStatus(XContext context, GetEnergyConsumptionAnalysisInput input) {
        int sumWellNumber = 0;
        int normallyOpenWellNumber = 0;
        int interOpenWellNumber = 0;
        BigDecimal liquidProduction = BigDecimal.ZERO;
        BigDecimal electricityConsumption = BigDecimal.ZERO;
        BigDecimal carbonReduction = BigDecimal.ZERO;
        BigDecimal economicBenefit = BigDecimal.ZERO;
        //查电站关联井口
        DynamicQueryBasePowerLineWellheadInput param = new DynamicQueryBasePowerLineWellheadInput();
        DynamicQueryBasePowerLineInput lineParam = new DynamicQueryBasePowerLineInput();
        if (CharSequenceUtil.isNotBlank(input.getStationName())) {
            //用stationName匹配电站
            param.setLineName(input.getStationName());
            lineParam.setLineNames(CollUtil.list(false, input.getStationName()));
        } else {
            List<String> orgIdsByPath = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            param.setOuIds(orgIdsByPath);
            lineParam.setOuIdList(orgIdsByPath);
        }
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = ServiceUtil.getPowerLineWellheadList(context, param);
        //计算井口数
        if (CollUtil.isNotEmpty(wellheadList)) {
            sumWellNumber = wellheadList.size();
            List<String> set = new ArrayList<>(wellheadList.size());
            for (DynamicQueryBasePowerLineWellheadViewOutput out : wellheadList) {
                set.add(out.getWellNumber());
                if (CharSequenceUtil.equals(out.getRunTypeKey(), BusinessConstant.INTERVAL_PUMPING_WELL)) {
                    normallyOpenWellNumber++;
                } else {
                    interOpenWellNumber++;
                }
            }
        }
        //查电站列表,计算统计数据
        List<DynamicQueryBasePowerLineOutput> powerLineList = ServiceUtil.getPowerLineList(context, lineParam);
        if (CollUtil.isNotEmpty(powerLineList)) {
            QueryWrapper<StationDailyProductionSituationEnt> queryWrapper = new QueryWrapper<>();
            if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
                queryWrapper.select("NVL( ROUND( AVG( DAILY_ELECTRICITY_CONSUMPTION ), 2 ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                        "NVL( ROUND( AVG( DAILY_LIQUID_PRODUCTION ), 2 ), 0 ) AS DAILY_LIQUID_PRODUCTION",
                        "NVL( ROUND( SUM( PHOTOVOLTAIC_POWER ), 2 ), 0 ) AS PHOTOVOLTAIC_POWER"
                );
            } else {
                queryWrapper.select("IFNULL( ROUND( AVG( DAILY_ELECTRICITY_CONSUMPTION ), 2 ), 0 ) AS DAILY_ELECTRICITY_CONSUMPTION",
                        "IFNULL( ROUND( AVG( DAILY_LIQUID_PRODUCTION ), 2 ), 0 ) AS DAILY_LIQUID_PRODUCTION",
                        "IFNULL( ROUND( SUM( PHOTOVOLTAIC_POWER ), 2 ), 0 ) AS PHOTOVOLTAIC_POWER"
                );
            }
            StationDailyProductionSituationMapper mapper = context.getBean(StationDailyProductionSituationMapper.class);
            StationDailyProductionSituationEnt ent = mapper.selectOne(queryWrapper.lambda()
                    .in(StationDailyProductionSituationEnt::getStationName, powerLineList.stream()
                            .map(DynamicQueryBasePowerLineOutput::getLineName)
                            .collect(Collectors.toSet()))
            );
            liquidProduction = ent.getDailyLiquidProduction();
            electricityConsumption = ent.getDailyElectricityConsumption();
            economicBenefit = ServiceUtil.getEconomicBenefitYuan(ent.getPhotovoltaicPower(), ServiceUtil.getOilFieldCode(context));
            carbonReduction = ServiceUtil.calculateCarbonReduction(ent.getPhotovoltaicPower());
        }
        return XSingleResult.success(GetWellProductionStatusOutput.builder()
                .sumWellNumber(sumWellNumber)
                .normallyOpenWellNumber(normallyOpenWellNumber)
                .interOpenWellNumber(interOpenWellNumber)
                .liquidProduction(liquidProduction)
                .electricityConsumption(electricityConsumption)
                .carbonReduction(carbonReduction)
                .economicBenefit(economicBenefit)
                .build());
    }

    /**
     * 井口实时信息
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetRealTimeInfoOutput }>
     */
    @XText("井组监控--井口实时信息")
    @XApiPost
    public XListResult<GetRealTimeInfoOutput> wellRealTimeInfo(XContext context, GetEnergyConsumptionAnalysisInput input) {
        List<GetRealTimeInfoOutput> currentList = Collections.emptyList();
        //查电站关联井口
        DynamicQueryBasePowerLineWellheadInput param = new DynamicQueryBasePowerLineWellheadInput();
        param.setRunTypeKey(input.getRunTypeKey());
        DynamicQueryBasePowerLineInput lineParam = new DynamicQueryBasePowerLineInput();
        if (CharSequenceUtil.isNotBlank(input.getStationName())) {
            //用stationName匹配电站
            param.setLineName(input.getStationName());
            lineParam.setLineNames(CollUtil.list(false, input.getStationName()));
        } else {
            List<String> orgIdsByPath = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            param.setOuIds(orgIdsByPath);
            lineParam.setOuIdList(orgIdsByPath);
        }
        List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList = ServiceUtil.getPowerLineWellheadList(context, param);
        if (CollUtil.isNotEmpty(wellheadList)) {
            String wellStatus = input.getWellStatus();
            Map<String, DynamicQueryBasePowerLineWellheadViewOutput> collect = new HashMap<>(wellheadList.size());
            List<String> wellIds = new ArrayList<>(wellheadList.size());
            for (DynamicQueryBasePowerLineWellheadViewOutput well : wellheadList) {
                collect.put(well.getWellNumber(), well);
                wellIds.add(well.getWellheadId());
            }
            //查井口运行状态
            ThirdCurrentWellConditionMapper mapper = context.getBean(ThirdCurrentWellConditionMapper.class);
            List<ThirdCurrentWellConditionEnt> list = mapper.selectList(new LambdaQueryWrapper<ThirdCurrentWellConditionEnt>()
                    .in(ThirdCurrentWellConditionEnt::getWellNumber, collect.keySet())
                    .eq(CharSequenceUtil.isNotBlank(wellStatus), ThirdCurrentWellConditionEnt::getWellStatus, wellStatus)
                    .eq(ThirdCurrentWellConditionEnt::getSystemSource, ServiceUtil.getOilFieldCode(context))
            );
            Map<String, ThirdCurrentWellConditionEnt> wellStatusMap;
            if (CollUtil.isNotEmpty(list)) {
                wellStatusMap = list.stream()
                        .collect(Collectors.toMap(ThirdCurrentWellConditionEnt::getWellNumber, Function.identity()));
            } else {
                wellStatusMap = Collections.emptyMap();
            }
            //当日间开
            ISpaceOptimizeShortCloudService service = context.getBean(ISpaceOptimizeShortCloudService.class);
            XListResult<GetSpaceOptimizeShortWellheadOutput> result = service.dailyOptimizationResult(context, GetSpaceOptimizeShortWellheadInput.builder()
                    .wellheadIds(wellIds)
                    .build());
            result.throwIfFail();
            List<GetSpaceOptimizeShortWellheadOutput> optimizeWellheadList = result.getResult();
            Map<String, List<SpaceOptimizeDateDuration>> listMap;
            if (CollUtil.isNotEmpty(optimizeWellheadList)) {
                listMap = optimizeWellheadList.stream()
                        .collect(Collectors.toMap(GetSpaceOptimizeShortWellheadOutput::getWellNumber, GetSpaceOptimizeShortWellheadOutput::getOptimizeList));
            } else {
                listMap = Collections.emptyMap();
            }
            //封装数据
            ThirdCurrentWellConditionEnt ent;
            BigDecimal nextOpen;
            DateTime now = DateUtil.date();
            String currentWellStatus;
            currentList = new ArrayList<>(list.size());
            for (Map.Entry<String, DynamicQueryBasePowerLineWellheadViewOutput> entry : collect.entrySet()) {
                if (wellStatusMap.containsKey(entry.getKey())) {
                    ent = wellStatusMap.get(entry.getKey());
                    currentWellStatus = ent.getWellStatus();
                    if (ObjectUtil.isNull(ent.getNextTime()) ||
                            CharSequenceUtil.contains(DateUtil.date(ent.getNextTime()).toString(), ThirdPartyApiConstant.DATE_FLAG) ||
                            DateUtil.compare(now, ent.getNextTime()) >= 0) {
                        nextOpen = BigDecimal.valueOf(-1);
                    } else {
                        nextOpen = BaseUtils.divide(BigDecimal.valueOf(DateUtil.between(ent.getNextTime(), now, DateUnit.MINUTE)), BusinessConstant.BIG_DECIMAL_60);
                    }
                } else {
                    nextOpen = BigDecimal.ZERO;
                    currentWellStatus = BusinessConstant.CLOSE_WELL;
                    if (!CharSequenceUtil.equals(currentWellStatus, input.getWellStatus())) {
                        continue;
                    }
                }
                currentList.add(
                        GetRealTimeInfoOutput.builder()
                                .wellNumber(entry.getKey())
                                .nextOpen(nextOpen)
                                .wellStatus(currentWellStatus)
                                .runTypeName(entry.getValue().getRunTypeName())
                                .ratedPower(entry.getValue().getRatedPower())
                                .optimizeList(listMap.getOrDefault(entry.getKey(), Collections.emptyList()))
                                .build()
                );
            }
        }
        return XListResult.success(currentList);
    }

    /*------------------------------ 能耗分析(松原) ------------------------------*/

    /**
     * 能耗分析(松原)--峰谷能耗对比分析
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetEnergyComparisonAnalysisOutput }>
     */
    @XText("能耗分析--峰谷能耗对比分析")
    @XApiGet
    public XListResult<GetEnergyComparisonAnalysisOutput> energyComparisonAnalysis(XContext context, QueryEnergyConsumptionAnalysisInput input) {
        DynamicQueryBasePowerLineWellheadInput wellheadInput = new DynamicQueryBasePowerLineWellheadInput();
        DynamicQueryBasePowerLinePlantInput plantInput = new DynamicQueryBasePowerLinePlantInput();
        String stationName = input.getStationName();
        if (CharSequenceUtil.isNotBlank(stationName)) {
            wellheadInput.setLineName(stationName);
            plantInput.setLineName(stationName);
        } else {
            List<String> orgIds = ServiceUtil.getOrgIdsByPath(context, input.getOuId());
            wellheadInput.setOuIds(orgIds);
            plantInput.setOuIds(orgIds);
        }
        input.getBetweenDate();
        BigDecimal electricityFees;
        BigDecimal divide;
        //封装数据
        List<DateTime> rangeToList = DateUtil.rangeToList(input.getStartTime(), input.getEndTime(), DateField.DAY_OF_YEAR);
        List<GetEnergyComparisonAnalysisOutput> outputs = new ArrayList<>(rangeToList.size());
        for (DateTime dateTime : rangeToList) {
            electricityFees = RandomUtil.randomBigDecimal(BusinessConstant.BIG_DECIMAL_60, BusinessConstant.BIG_DECIMAL_10000);
            divide = BaseUtils.divide(electricityFees, BigDecimal.TEN);
            outputs.add(GetEnergyComparisonAnalysisOutput.builder()
                    .dateFormat(dateTime.toString())
                    .electricityFees(electricityFees.setScale(2, RoundingMode.HALF_UP))
                    .valleyElectricPower(divide.multiply(BusinessConstant.BIG_DECIMAL_4))
                    .ordinaryElectricPower(divide.multiply(BusinessConstant.BIG_DECIMAL_3))
                    .highElectricPower(divide.multiply(BusinessConstant.BIG_DECIMAL_2))
                    .peakElectricPower(divide)
                    .build());
        }
        return XListResult.success(outputs);
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 电量统计
     *
     * @param context         上下文
     * @param startTime       开始时间
     * @param endTime         结束时间
     * @param stationNameList 车站名称列表
     * @param flag            旗帜 (1_发电,2_用电)
     * @param oilFieldCode    部署环境
     * @param dateType        日期范围类型(0_本月;1_本年)
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> getPowerMap(XContext context, Date startTime, Date endTime, List<String> stationNameList, int flag, String oilFieldCode, Integer dateType) {
        Map<Date, BigDecimal> powerMap;
        switch (oilFieldCode) {
            case BusinessConstant.ENV_SY:
                StationDailyProductionSituationMapper situationMapper = context.getBean(StationDailyProductionSituationMapper.class);
                QueryWrapper<StationDailyProductionSituationEnt> queryWrapper = new QueryWrapper<>();
                if (CharSequenceUtil.equals(context.getProperty("x.db.databaseId"), BusinessConstant.ORACLE)) {
                    queryWrapper.select("NVL( ROUND( SUM( daily_electricity_consumption ), 2 ), 0 ) AS daily_electricity_consumption",
                            "NVL( ROUND( SUM( daily_liquid_production ), 2 ), 0 ) AS daily_liquid_production",
                            "NVL( ROUND( SUM( photovoltaic_power ), 2 ), 0 ) AS photovoltaic_power",
                            "data_date"
                    );
                } else {
                    queryWrapper.select("IFNULL( ROUND( SUM( daily_electricity_consumption ), 2 ), 0 ) AS daily_electricity_consumption",
                            "IFNULL( ROUND( SUM( daily_liquid_production ), 2 ), 0 ) AS daily_liquid_production",
                            "IFNULL( ROUND( SUM( photovoltaic_power ), 2 ), 0 ) AS photovoltaic_power",
                            "data_date"
                    );
                }
                List<StationDailyProductionSituationEnt> list = situationMapper.selectList(queryWrapper.lambda()
                        .in(CollUtil.isNotEmpty(stationNameList), StationDailyProductionSituationEnt::getStationName, stationNameList)
                        .between(ObjectUtil.isAllNotEmpty(startTime, endTime), StationDailyProductionSituationEnt::getDataDate, startTime, endTime)
                        .groupBy(StationDailyProductionSituationEnt::getDataDate)
                );
                if (CollUtil.isNotEmpty(list)) {
                    powerMap = list.stream()
                            .collect(Collectors.toMap(StationDailyProductionSituationEnt::getDataDate, flag == 1 ? StationDailyProductionSituationEnt::getPhotovoltaicPower : StationDailyProductionSituationEnt::getDailyElectricityConsumption));
                } else {
                    powerMap = Collections.emptyMap();
                }
                break;
            default:
                ThirdDailyAccumulationUpdateViewMapper mapper = context.getBean(ThirdDailyAccumulationUpdateViewMapper.class);
                List<ThirdDailyAccumulationUpdateView> powerList = mapper.selectPowerTrendByDate(ThirdDailyAccumulationUpdateView.builder()
                        .startTime(startTime)
                        .endTime(endTime)
                        .stationNameList(stationNameList)
                        .systemSource(oilFieldCode)
                        .dayFlag(dateType)
                        .build());
                if (CollUtil.isNotEmpty(powerList) && CollUtil.isNotEmpty(stationNameList)) {
                    powerMap = powerList.stream()
                            .collect(Collectors.toMap(ThirdDailyAccumulationUpdateView::getCreateDate, flag == 1 ? ThirdDailyAccumulationUpdateView::getPhotovoltaicPower : ThirdDailyAccumulationUpdateView::getDailyElectricityConsumption));
                } else {
                    powerMap = Collections.emptyMap();
                }
        }
        return powerMap;
    }

    /**
     * 获取井口列表
     *
     * @param context 上下文
     * @param ouIds   ou-ids
     * @return {@link List }<{@link DynamicQueryBaseWellheadOutput }>
     */
    private List<DynamicQueryBaseWellheadOutput> getWellListByOuId(XContext context, List<String> ouIds) {
        return ServiceUtil.getBaseWellheadList(context, DynamicQueryBaseWellheadInput.builder()
                .ouIds(ouIds)
                .build());
    }

    /**
     * 查询用电功率
     *
     * @param context      上下文
     * @param wellheadList 井口一览表
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @param oilFieldCode 环境
     * @param dayFlag      日期范围(1_小时;2_日;_3_月)
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> queryPowerConsumptionHour(XContext context, List<DynamicQueryBasePowerLineWellheadViewOutput> wellheadList, Date startTime, Date endTime, String oilFieldCode, int dayFlag) {
        Map<Date, BigDecimal> avgMap;
        if (CollUtil.isNotEmpty(wellheadList)) {
            ThirdWellAvgActivePowerViewMapper mapper = context.getBean(ThirdWellAvgActivePowerViewMapper.class);
            avgMap = mapper.selectPowerConsumptionHour(ThirdWellAvgActivePowerView.builder()
                            .wellNumbers(wellheadList.stream()
                                    .map(DynamicQueryBasePowerLineWellheadViewOutput::getWellNumber)
                                    .collect(Collectors.toList()))
                            .startTime(startTime)
                            .endTime(endTime)
                            .systemSource(oilFieldCode)
                            .dayFlag(dayFlag)
                            .build())
                    .stream()
                    .collect(Collectors.toMap(ThirdWellAvgActivePowerView::getInputTime, ThirdWellAvgActivePowerView::getAvgActivePower));
        } else {
            avgMap = Collections.emptyMap();
        }
        return avgMap;
    }

    /**
     * 查询发电功率
     *
     * @param context      上下文
     * @param plantList    工厂清单
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @param oilFieldCode 环境
     * @param dayFlag      日期范围(1_小时;2_日;_3_月)
     * @return {@link Map }<{@link Date }, {@link BigDecimal }>
     */
    private Map<Date, BigDecimal> queryGeneratedPowerHour(XContext context, List<DynamicQueryBasePowerLinePlantViewOutput> plantList, Date startTime, Date endTime, String oilFieldCode, int dayFlag) {
        Map<Date, BigDecimal> powerMap;
        if (CollUtil.isNotEmpty(plantList)) {
            ThirdActivePowerViewMapper dailyUpdateMapper = context.getBean(ThirdActivePowerViewMapper.class);
            powerMap = dailyUpdateMapper.selectGeneratedPowerHour(ThirdActivePowerView.builder()
                            .stationNames(plantList.stream()
                                    .map(DynamicQueryBasePowerLinePlantViewOutput::getStationName)
                                    .collect(Collectors.toList()))
                            .startTime(startTime)
                            .endTime(endTime)
                            .systemSource(oilFieldCode)
                            .dayFlag(dayFlag)
                            .build())
                    .stream()
                    .collect(Collectors.toMap(ThirdActivePowerView::getCreateDate, ThirdActivePowerView::getPhotovoltaicPower));
        } else {
            powerMap = Collections.emptyMap();
        }
        return powerMap;
    }

    /**
     * 设置功率分析列表
     *
     * @param avgMap    平均地图
     * @param powerMap  功率图
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @param dayFlag   日期范围(1_小时;2_日;_3_月)
     * @return {@link List }<{@link GetPowerAnalyseOutput }>
     */
    private List<GetPowerAnalyseOutput> setPowerAnalyseList(Map<Date, BigDecimal> avgMap, Map<Date, BigDecimal> powerMap, Date startTime, Date endTime, String oilFieldCode, int dayFlag) {
        //封装数据
        DateField dateField;
        switch (dayFlag) {
            case 2:
                dateField = DateField.DAY_OF_YEAR;
                break;
            case 3:
                dateField = DateField.MONTH;
                break;
            default:
                dateField = DateField.HOUR_OF_DAY;
        }
        List<DateTime> rangeToList = DateUtil.rangeToList(startTime, endTime, dateField);
        List<GetPowerAnalyseOutput> outputs = new ArrayList<>(rangeToList.size());
        BigDecimal powerConsumption;
        BigDecimal powerGeneration;
        BigDecimal defaultPower = CharSequenceUtil.equals(oilFieldCode, BusinessConstant.ENV_CQ) ? BigDecimal.ZERO : null;
        for (DateTime dateTime : rangeToList) {
            if (avgMap.containsKey(dateTime)) {
                //用电*3
                powerConsumption = avgMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerConsumption = defaultPower;
            }
            if (ObjectUtil.isNotNull(powerConsumption) && powerConsumption.compareTo(BigDecimal.ZERO) > 0) {
                powerConsumption = powerConsumption.multiply(BigDecimal.valueOf(3));
            }
            if (powerMap.containsKey(dateTime)) {
                powerGeneration = powerMap.get(dateTime).setScale(2, RoundingMode.HALF_UP);
            } else {
                powerGeneration = BigDecimal.ZERO;
            }
            outputs.add(GetPowerAnalyseOutput.builder()
                    .dateFormat(dateTime.toString())
                    .powerConsumption(powerConsumption)
                    .powerGeneration(powerGeneration)
                    .build());
        }
        return outputs;
    }

    /**
     * 用电趋势统计
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link Map }<{@link Date }, {@link GetLineDailyElectricityTrendOutput }>
     */
    private Map<Date, GetLineDailyElectricityTrendOutput> getDailyElectricityTrendMap(XContext context, GetLineDailyElectricityTrendInput input) {
        IDailyElectricityTrendCloudService service = context.getBean(IDailyElectricityTrendCloudService.class);
        XListResult<GetLineDailyElectricityTrendOutput> result = service.queryDailyElectricityTrend(context, input);
        result.throwIfFail();
        List<GetLineDailyElectricityTrendOutput> list = result.getResult();
        Map<Date, GetLineDailyElectricityTrendOutput> collect;
        if (CollUtil.isNotEmpty(list)) {
            collect = list.stream()
                    .collect(Collectors.toMap(GetLineDailyElectricityTrendOutput::getCreateDate, Function.identity()));
        } else {
            collect = Collections.emptyMap();
        }
        return collect;
    }

    /**
     * 计算电费
     *
     * @param photovoltaicPower 光伏发电
     * @return {@link BigDecimal }
     */
    private BigDecimal calculateElectricityFees(BigDecimal photovoltaicPower) {
        BigDecimal economicBenefit;
        if (photovoltaicPower.compareTo(BigDecimal.ZERO) > 0) {
            economicBenefit = photovoltaicPower
                    .multiply(BigDecimal.valueOf(0.6669))
                    .setScale(2, RoundingMode.HALF_UP);
        } else {
            economicBenefit = BigDecimal.ZERO;
        }
        return economicBenefit;
    }
}