package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadInput;
import pps.cloud.base.service.data.base_wellhead.DynamicQueryBaseWellheadOutput;
import pps.cloud.middle.service.IWellDailyProductionSituationCloudService;
import pps.cloud.middle.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationInput;
import pps.cloud.middle.service.data.wellhead_daily_production_situation.GetWellheadDailyProductionSituationOutput;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.DynamicQueryPlantPredictedPowerOutput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewOutput;
import pps.core.auth.HttpRequestClient;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.PlantPredictedPowerShortTermView;
import pps.core.prediction.entity.PredictedPowerView;
import pps.core.prediction.entity.WellheadDailyProductionSituationView;
import pps.core.prediction.mapper.PlantPredictedPowerShortTermViewMapper;
import pps.core.prediction.mapper.PredictedPowerViewMapper;
import pps.core.prediction.mapper.WellheadDailyProductionSituationViewMapper;
import pps.core.prediction.service.data.wellhead_daily_production_situation.WellheadDailyProductionSituationJob;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 光伏预测Cloud模块
 *
 * @author ZWT
 * @date 2023/09/13 19:05
 */
@XService
public class PlantPredictedPowerCloudServiceImpl implements IPlantPredictedPowerCloudService {

    /**
     * 光伏预测Cloud模块--条件查询时段发电量平均值
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        List<String> plantIds = input.getPlantIds();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        Integer formatType = input.getFormatType();
        List<PredictedPowerView> list = null;
        if (CollUtil.isNotEmpty(plantIds)) {
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            switch (dateType) {
                case 0:
                    list = mapper.selectMidPowerList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .formatType(formatType)
                            .build());
                    break;
                case 1:
                    list = mapper.selectShortPowerList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .formatType(formatType)
                            .build());
                    break;
                default:
            }
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePower(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        List<String> plantIds = input.getPlantIds();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List<PredictedPowerView> list = null;
        if (CollUtil.isNotEmpty(plantIds)) {
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            switch (dateType) {
                case 0:
                    list = mapper.selectMidPowerHalfHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .formatType(input.getFormatType())
                            .build());
                    break;
                case 1:
                    list = mapper.selectShortPowerHalfHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .formatType(input.getFormatType())
                            .build());
                    break;
                default:
            }
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    /**
     * 光伏预测Cloud模块--获取每小时平均发电量列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQueryPlantPredictedPowerOutput}>
     */
    @Override
    public XListResult<DynamicQueryPlantPredictedPowerOutput> queryAveragePowerGenerationHourListByParam(XContext context, DynamicQueryPlantPredictedPowerInput input) {
        List<String> plantIds = input.getPlantIds();
        String startTime = input.getStartTime();
        String endTime = input.getEndTime();
        Integer dateType = input.getDateType();
        List list = null;
        if (CollUtil.isNotEmpty(plantIds)) {
            PredictedPowerViewMapper mapper = context.getBean(PredictedPowerViewMapper.class);
            switch (dateType) {
                case 0:
                    list = mapper.selectMidPowerHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                case 1:
                    list = mapper.selectShortPowerHourList(PredictedPowerView.builder()
                            .plantIds(plantIds)
                            .startTime(startTime)
                            .endTime(endTime)
                            .build());
                    break;
                default:
            }
        }
        List<DynamicQueryPlantPredictedPowerOutput> outputs;
        if (CollUtil.isEmpty(list)) {
            outputs = Collections.emptyList();
        } else {
            outputs = XCopyUtils.copyNewList(list, DynamicQueryPlantPredictedPowerOutput.class);
        }
        return XListResult.success(outputs);
    }

    /**
     * 光伏预测Cloud模块--每30分钟一天预测发电量
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link GetPredictedPowerViewOutput }>
     */
    @Override
    public XListResult<GetPredictedPowerViewOutput> queryPredictionPower30(XContext context, GetPredictedPowerViewInput input) {
        PlantPredictedPowerShortTermViewMapper mapper = context.getBean(PlantPredictedPowerShortTermViewMapper.class);
        return XListResult.success(XCopyUtils.copyNewList(mapper.selectPredictionPower30(PlantPredictedPowerShortTermView.builder()
                        .stationNames(input.getStationNames())
                        .startTime(input.getDataDate())
                        .endTime(DateUtil.offsetDay(input.getDataDate(), 1))
                        .build()),
                GetPredictedPowerViewOutput.class));
    }

    /**
     * 日耗电日产液信息定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult wellTechDailyJob(XContext context) {
        List<WellheadDailyProductionSituationJob> list = null;
        //区分环境
        String oilFieldCode = ServiceUtil.getOilFieldCode(context);
        switch (oilFieldCode) {
            case BusinessConstant.ENV_SY:
                IWellDailyProductionSituationCloudService service = context.getBean(IWellDailyProductionSituationCloudService.class);
                XListResult<GetWellheadDailyProductionSituationOutput> result = service.getWellDailyProductionSituationList(context, GetWellheadDailyProductionSituationInput.builder()
                        .dataDate(DateUtil.beginOfDay(DateUtil.yesterday()).toDateStr())
                        .build());
                result.throwIfFail();
                List<GetWellheadDailyProductionSituationOutput> dailyList = result.getResult();
                if (CollUtil.isNotEmpty(dailyList)) {
                    list = new ArrayList<>(dailyList.size());
                    for (GetWellheadDailyProductionSituationOutput output : dailyList) {
                        list.add(
                                WellheadDailyProductionSituationJob.builder()
                                        .wellNumber(output.getWellNumber())
                                        .dataDate(DateUtil.parseDate(output.getDataDate()))
                                        .dailyLiquidProduction(output.getDailyLiquidProduction())
                                        .productionTime(output.getProductionTime())
                                        .dailyElectricityConsumption(output.getDailyElectricityConsumption())
                                        .dailyUtilityElectricityConsumption(output.getDailyUtilityElectricityConsumption())
                                        .electricityConsumptionPh(output.getElectricityConsumptionPh())
                                        .electricityConsumptionHh(output.getElectricityConsumptionHh())
                                        .electricityConsumptionNp(output.getElectricityConsumptionNp())
                                        .electricityConsumptionVh(output.getElectricityConsumptionVh())
                                        .electricityChargeCity(output.getElectricityChargeCity())
                                        .electricityChargesPh(output.getElectricityChargesPh())
                                        .peakHourTariff(output.getPeakHourTariff())
                                        .normalPeriodTariff(output.getNormalPeriodTariff())
                                        .alleyHoursTariff(output.getAlleyHoursTariff())
                                        .build()
                        );
                    }
                }
                break;
            case BusinessConstant.ENV_CQ:
                String token = ServiceUtil.getToken(context);
                HttpRequestClient client = new HttpRequestClient(token);
                Map<String, Object> param = new HashMap<>(4);
                param.put("Calc_Date", DateUtil.yesterday().toString(BusinessConstant.DATE_FORMAT_DAY));
                String url = XStartup.getCurrent().getProperty("third-party.chang-qing.url");
                url += ThirdPartyApiConstant.CQ_WELL_TECH_DAILY;
                String resultStr = client.doPostForm(url, param);
                context.getLogger().info("*** CQ_WELL_TECH_DAILY : {}", resultStr);
                JSONObject jsonObject = JSON.parseObject(resultStr);
                String data = jsonObject.getString("data");
                list = JSON.parseArray(data, WellheadDailyProductionSituationJob.class);
                break;
            default:
        }
        if (CollUtil.isEmpty(list)) {
            context.getLogger().info("------ WellTechDailyJob 未发现数据: {}", DateUtil.yesterday().toDateStr());
            return XServiceResult.OK;
        }
        List<String> collect = list.stream()
                .map(WellheadDailyProductionSituationJob::getWellNumber)
                .collect(Collectors.toList());
        Map<String, String> map = ServiceUtil.getBaseWellheadList(context, DynamicQueryBaseWellheadInput.builder()
                        .wellNumbers(collect)
                        .build()).stream()
                .collect(Collectors.toMap(DynamicQueryBaseWellheadOutput::getWellNumber, DynamicQueryBaseWellheadOutput::getId));
        DateTime now = DateUtil.date();
        List<WellheadDailyProductionSituationView> dtoList = new ArrayList<>(list.size());
        for (WellheadDailyProductionSituationJob wellhead : list) {
            dtoList.add(WellheadDailyProductionSituationView.builder()
                    .id(BaseUtils.randomUUIDString())
                    .wellNumber(wellhead.getWellNumber())
                    .dataDate(wellhead.getDataDate())
                    .dailyLiquidProduction(ObjectUtil.defaultIfNull(wellhead.getDailyLiquidProduction(), BigDecimal.ZERO))
                    .productionTime(ObjectUtil.defaultIfNull(wellhead.getProductionTime(), BigDecimal.ZERO))
                    .dailyElectricityConsumption(ObjectUtil.defaultIfNull(wellhead.getDailyElectricityConsumption(), BigDecimal.ZERO))
                    .wellheadId(map.get(wellhead.getWellNumber()))
                    .dailyUtilityElectricityConsumption(ObjectUtil.defaultIfNull(wellhead.getDailyUtilityElectricityConsumption(), BigDecimal.ZERO))
                    .electricityConsumptionPh(ObjectUtil.defaultIfNull(wellhead.getElectricityConsumptionPh(), BigDecimal.ZERO))
                    .electricityConsumptionHh(ObjectUtil.defaultIfNull(wellhead.getElectricityConsumptionHh(), BigDecimal.ZERO))
                    .electricityConsumptionNp(ObjectUtil.defaultIfNull(wellhead.getElectricityConsumptionNp(), BigDecimal.ZERO))
                    .electricityConsumptionVh(ObjectUtil.defaultIfNull(wellhead.getElectricityConsumptionVh(), BigDecimal.ZERO))
                    .electricityChargeCity(ObjectUtil.defaultIfNull(wellhead.getElectricityChargeCity(), BigDecimal.ZERO))
                    .electricityChargesPh(ObjectUtil.defaultIfNull(wellhead.getElectricityChargesPh(), BigDecimal.ZERO))
                    .peakHourTariff(ObjectUtil.defaultIfNull(wellhead.getPeakHourTariff(), BigDecimal.ZERO))
                    .normalPeriodTariff(ObjectUtil.defaultIfNull(wellhead.getNormalPeriodTariff(), BigDecimal.ZERO))
                    .alleyHoursTariff(ObjectUtil.defaultIfNull(wellhead.getAlleyHoursTariff(), BigDecimal.ZERO))
                    .systemSource(oilFieldCode)
                    .createDate(now)
                    .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            WellheadDailyProductionSituationViewMapper mapper = context.getBean(WellheadDailyProductionSituationViewMapper.class);
            if (CollUtil.isNotEmpty(dtoList)) {
                if (dtoList.size() > BaseUtils.BATCH_SIZE) {
                    List<List<WellheadDailyProductionSituationView>> subList = BaseUtils.getSubList(dtoList);
                    subList.forEach(mapper::batchInsertList);
                } else {
                    mapper.batchInsertList(dtoList);
                }
            }
            return XServiceResult.OK;
        });
    }
}
