package pps.core.prediction.service;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.common.constant.BusinessConstant;
import pps.core.prediction.entity.PlantPredictedPowerDataEnt;
import pps.core.prediction.entity.PlantPredictedPowerLongTermDataEnt;
import pps.core.prediction.mapper.PlantPredictedPowerDataMapper;
import pps.core.prediction.mapper.PlantPredictedPowerLongTermDataMapper;
import pps.core.prediction.service.data.plant_predicted_power_data.DoPlantPredictedPowerDataInput;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPlantPredictedPowerDataInput;
import pps.core.prediction.service.data.plant_predicted_power_data.GetPlantPredictedPowerDataOutput;
import pps.core.prediction.utils.HttpUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


@XService
@XText("电站预测发电数据表")
public class PlantPredictedPowerDataService {

    @XApiGet
    @XText("根据电站预测4小时内的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getFourPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input) {
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        if (CharSequenceUtil.isBlank(input.getStartTime())) {
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, date)
                    .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetHour(date, 4));
        } else {
            String dataDate = DateUtil.formatDate(date) + " " + input.getStartTime() + BusinessConstant.INITIALIZATION_SECOND;
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, dataDate)
                    .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetHour(DateUtil.parseDateTime(dataDate), 4));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerDataEnt::getDataDate);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerDataOutput.class);
        outputs.forEach(item -> {
            item.setPredictedPower(item.getPower());
        });
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测7天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getSevenPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input) {
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId());
        if (CharSequenceUtil.isBlank(input.getStartTime())) {
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, date)
                    .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetDay(date, 7));
        } else {
            queryWrapper.lambda().ge(PlantPredictedPowerDataEnt::getDataDate, input.getStartTime())
                    .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetDay(DateUtil.parseDateTime(input.getStartTime()), 7));
        }
        queryWrapper.lambda().orderByAsc(PlantPredictedPowerDataEnt::getDataDate);
        List<PlantPredictedPowerDataEnt> list = mapper.selectList(queryWrapper);
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerDataOutput.class);
        outputs.forEach(item -> {
            item.setPredictedPower(item.getPower());
        });
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测30天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getThirtyPlantPredictedPowerData_delete(XContext context, GetPlantPredictedPowerDataInput input) {
        //今年（7天总值）/去年（7天的总值）  同比值
        PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
        Date date = new Date();
        QueryWrapper<PlantPredictedPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("IFNULL(avg(power ) , 0 ) power")
                .lambda()
                .eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId())
                .ge(PlantPredictedPowerDataEnt::getDataDate, date)
                .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetDay(date, 7));
        PlantPredictedPowerDataEnt currentData = mapper.selectOne(queryWrapper);
        queryWrapper.clear();
        DateTime lastYear = DateUtil.offset(date, DateField.YEAR, -1);
        queryWrapper.select("IFNULL(avg(power ) , 0 ) power")
                .lambda()
                .eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId())
                .ge(PlantPredictedPowerDataEnt::getDataDate, lastYear)
                .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetDay(lastYear, 7));
        PlantPredictedPowerDataEnt lastData = mapper.selectOne(queryWrapper);
        BigDecimal compare = new BigDecimal(1);
        if (lastData.getPower().compareTo(BigDecimal.ZERO) > 0) {
            compare = currentData.getPower().divide(lastData.getPower(), 6, BigDecimal.ROUND_HALF_UP);
        }
        queryWrapper.clear();
        queryWrapper.select("data_date, power")
                .lambda()
                .eq(PlantPredictedPowerDataEnt::getPlantId, input.getPlantId())
                .ge(PlantPredictedPowerDataEnt::getDataDate, lastYear)
                .le(PlantPredictedPowerDataEnt::getDataDate, DateUtil.offsetDay(lastYear, 30));
        List<PlantPredictedPowerDataEnt> lastList = mapper.selectList(queryWrapper);
        //获取去年30天的数据，预测今年30天的数据
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(lastList, GetPlantPredictedPowerDataOutput.class);
        for (GetPlantPredictedPowerDataOutput item : outputs) {
            item.setPredictedPower(item.getPower().multiply(compare));
            item.setDataDate(DateUtil.formatDate(DateUtil.offset(DateUtil.parseDateTime(item.getDataDate()), DateField.YEAR, 1)));
        }
        return XListResult.success(outputs);
    }

    @XApiGet
    @XText("根据电站预测30天的预测数据")
    public XListResult<GetPlantPredictedPowerDataOutput> getThirtyPlantPredictedPowerData(XContext context, GetPlantPredictedPowerDataInput input) {
        PlantPredictedPowerLongTermDataMapper mapper = context.getBean(PlantPredictedPowerLongTermDataMapper.class);
        Date date = new Date();
        List<PlantPredictedPowerLongTermDataEnt> list = mapper.selectList(new QueryWrapper<PlantPredictedPowerLongTermDataEnt>().select("id, plant_id, data_date, temperature, humidity, wind_speed, wind_direction, pressure, plane_irradiance, horizontal_irradiance, power as predicted_power, create_time")
                .lambda()
                .eq(PlantPredictedPowerLongTermDataEnt::getPlantId, input.getPlantId())
                .ge(PlantPredictedPowerLongTermDataEnt::getDataDate, date)
                .le(PlantPredictedPowerLongTermDataEnt::getDataDate, DateUtil.offsetMonth(date, 1)));
        List<GetPlantPredictedPowerDataOutput> outputs = XCopyUtils.copyNewList(list, GetPlantPredictedPowerDataOutput.class);
        return XListResult.success(outputs);
    }

    @XText("test")
    @XApiPost(anonymous = true)
    public XServiceResult test(XContext context, DoPlantPredictedPowerDataInput input) {
        ThirdDataAccessCloudServiceImpl bean = context.getBean(ThirdDataAccessCloudServiceImpl.class);
        bean.test(context);
        bean.test2(context);
        PlantPredictedPowerCloudServiceImpl bean1 = context.getBean(PlantPredictedPowerCloudServiceImpl.class);
        bean1.test(context);
        StationDailyCloudServiceImpl bean2 = context.getBean(StationDailyCloudServiceImpl.class);
        bean2.test(context);
        return XServiceResult.OK;
    }

    @XText("预测电站发电数据")
    @XApiPost
    public XServiceResult doPredictedPowerData(XContext context, DoPlantPredictedPowerDataInput input) {
        doPredictedData(context, input.getPlantId());
        return XServiceResult.OK;
    }

    public void doPredictedData(XContext context, String plantId) {
        try {
            Object ret = HttpUtils.send2("http://127.0.0.1:10098/aiprediction/xgbreason?plantId=" + plantId,
                    "");
            context.getLogger().info(ret.toString());
        } catch (Exception e) {
            throw new RuntimeException("调用训练接口失败");
        }
    }
}