package pps.core.prediction.service.data.energy_consumption_analysis;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.Data;
import pps.core.prediction.enums.BusinessError;
import xstartup.base.exception.XServiceException;

import java.util.Date;

/**
 * 入参
 *
 * @author ZWT
 * @date 2024/10/20 20:36
 */
@Data
public class GetStatisticsInput {

    /**
     * 组织机构id
     */
    private String ouId;

    /**
     * 日期类型(1_本周;2_本月;3_本年)
     */
    private Integer dateType;

    /**
     * 开始时间
     */
    private Date beginTime;

    /**
     * 结束时间
     */
    private Date endTime;

    /**
     * 初始化开始结束日期
     * 1_本周;2_本月;3_本年
     */
    public void getBetweenDate() {
        if (ObjectUtil.isAllEmpty(this.beginTime, this.endTime)) {
            DateTime now = DateUtil.date();
            switch (ObjectUtil.defaultIfNull(this.dateType, 1)) {
                case 1:
                    this.beginTime = DateUtil.beginOfWeek(now);
                    this.endTime = DateUtil.endOfWeek(now);
                    break;
                case 2:
                    this.beginTime = DateUtil.beginOfMonth(now);
                    this.endTime = DateUtil.endOfMonth(now);
                    break;
                case 3:
                    this.beginTime = DateUtil.beginOfYear(now);
                    this.endTime = DateUtil.endOfYear(now);
                    break;
                default:
                    throw new XServiceException(BusinessError.DateTypeError);
            }
        } else {
            this.beginTime = DateUtil.beginOfDay(this.beginTime);
            this.endTime = DateUtil.endOfDay(this.endTime);
            this.dateType = 1;
        }
    }
}
